@@ -282,7 +282,7 @@ appear."
           (setf binding :function
                 localp nil
                 ftype (when (eq :declared (info :function :where-from name))
-                        (proclaimed-ftype name))
+                        (global-ftype name))
                 inlinep (info :function :inlinep name))))))
     (values binding
             localp
@@ -403,7 +403,7 @@ appear."
          (sb-c::lambda-var
           (setf binding :lexical
                 localp t
-                ignorep (sb-c::lambda-var-ignorep var)))
+                ignorep (sb-c:lambda-var-ignorep var)))
          ;; FIXME: IGNORE doesn't make sense for specials or constants
          ;; -- though it is _possible_ to declare them ignored, but
          ;; we don't keep the information around.
@@ -532,7 +532,7 @@ is referred to by the expression."
   (let ((env (if environment
                  (sb-c::make-restricted-lexenv environment)
                  (make-null-lexenv))))
-    (compile-in-lexenv lambda-expression env)))
+    (compile-in-lexenv lambda-expression env nil nil nil nil nil)))
 
 ;;; Add a bit of user-data to a lexenv.
 ;;;
