@@ -1810,7 +1810,7 @@ function to be removed without further warning."
        ;; However when it comes to actually executing the toplevel forms
        ;; that were compiled into thunks of target code to invoke,
        ;; all the known good entries must be preserved.
-       (substitute #',error-name 0 ,table-name)
+       (nsubstitute #',error-name 0 ,table-name)
 
        ,@(loop for info across *specialized-array-element-type-properties*
                for typecode = (saetp-typecode info)
@@ -1828,11 +1828,12 @@ function to be removed without further warning."
          (check-type (first args) symbol)
          (let ((tag (gensym "TAG")))
            `(funcall
-             (the function
+             (truly-the function
                (let ((,tag 0))
                  (when (%other-pointer-p ,(first args))
                    (setf ,tag (%other-pointer-widetag ,(first args))))
-                 (svref ,',table-name ,tag)))
+                 (svref (truly-the (simple-vector 256) (load-time-value ,',table-name t))
+                        ,tag)))
              ,@args))))))
 
 (defun sb-kernel::check-array-shape (array dimensions)
@@ -1858,15 +1859,24 @@ function to be removed without further warning."
                                      (initial-element nil element-p))
   (when (and element-p contents-p)
     (error "Can't specify both :INITIAL-ELEMENT and :INITIAL-CONTENTS"))
-  (let ((v (if contents-p
-               (make-array length :initial-contents initial-contents)
-               (make-array length :initial-element
-                           ;; 0 is the usual default, but NIL makes more sense
-                           ;; for weak vectors because it's the value assigned
-                           ;; when a pointer is broken by GC.
-                           (if element-p initial-element nil)))))
-    (set-header-data v vector-weak-subtype)
-    v))
+  ;; Explicitly compute a widetag with the weakness bit ORed in.
+  (let ((type (logior (ash vector-weak-subtype sb-vm:n-widetag-bits)
+                      sb-vm:simple-vector-widetag)))
+    ;; These allocation calls are the transforms of MAKE-ARRAY for a vector with
+    ;; the respective initializing keyword arg. This is badly OAOO-violating and
+    ;; almost makes me want to cry, but not quite enough for me to improve it.
+    (if contents-p
+        (let ((contents-length (length initial-contents)))
+          (if (= (truly-the index length) contents-length)
+              (replace (truly-the simple-vector (allocate-vector type length length))
+                       initial-contents)
+              (error "~S has ~D elements, vector length is ~D."
+                     :initial-contents contents-length length)))
+        (fill (truly-the simple-vector
+                         (allocate-vector type (truly-the index length) length))
+              ;; 0 is the usual default, but NIL makes more sense for weak vectors
+              ;; as it is the value assigned to broken hearts.
+              (if element-p initial-element nil)))))
 
 (defun weak-vector-p (x)
   (and (simple-vector-p x)
