@@ -10,10 +10,10 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;; HASH-TABLE is implemented as a STRUCTURE-OBJECT.
-(sb!xc:defstruct (hash-table (:copier nil)
+(sb-xc:defstruct (hash-table (:copier nil)
                              (:constructor %make-hash-table
                                (test
                                 test-fun
@@ -22,11 +22,10 @@
                                 rehash-threshold
                                 rehash-trigger
                                 table
-                                weakness
                                 index-vector
                                 next-vector
                                 hash-vector
-                                synchronized-p)))
+                                flags)))
   ;; The type of hash table this is. Only used for printing and as
   ;; part of the exported interface.
   (test nil :type symbol :read-only t)
@@ -40,10 +39,10 @@
   ;; How much to grow the hash table by when it fills up. If an index,
   ;; then add that amount. If a floating point number, then multiply
   ;; it by that.
-  (rehash-size nil :type (or index (single-float (1.0)))
+  (rehash-size nil :type (or index (single-float ($1.0)))
                :read-only t)
   ;; How full the hash table has to get before we rehash
-  (rehash-threshold nil :type (single-float (0.0) 1.0) :read-only t)
+  (rehash-threshold nil :type (single-float ($0.0) $1.0) :read-only t)
   ;; The number of entries before a rehash, just one less than the
   ;; size of the next-vector, hash-vector, and half the size of the
   ;; kv-vector.
@@ -55,10 +54,9 @@
   ;; This slot is used to link weak hash tables during GC. When the GC
   ;; isn't running it is always NIL.
   (next-weak-hash-table nil :type null)
-  ;; Non-NIL if this is some kind of weak hash table. For details see
-  ;; the docstring of MAKE-HASH-TABLE.
-  (weakness nil :type (member nil :key :value :key-or-value :key-and-value)
-            :read-only t)
+  ;; flags: WEAKNESS-KIND | FINALIZERSP | SYNCHRONIZEDP | WEAKP
+  ;; WEAKNESS-KIND is 2 bits, the rest are 1 bit each
+  (flags 0 :type (unsigned-byte 5) :read-only t)
   ;; Index into the Next vector chaining together free slots in the KV
   ;; vector.
   (next-free-kv 0 :type index)
@@ -67,34 +65,30 @@
   (cache nil :type (or null index))
   ;; The index vector. This may be larger than the hash size to help
   ;; reduce collisions.
-  (index-vector nil :type (simple-array sb!vm:word (*)))
+  (index-vector nil :type (simple-array sb-vm:word (*)))
   ;; This table parallels the KV vector, and is used to chain together
   ;; the hash buckets and the free list. A slot will only ever be in
   ;; one of these lists.
-  (next-vector nil :type (simple-array sb!vm:word (*)))
+  (next-vector nil :type (simple-array sb-vm:word (*)))
   ;; This table parallels the KV table, and can be used to store the
   ;; hash associated with the key, saving recalculation. Could be
   ;; useful for EQL, and EQUAL hash tables. This table is not needed
   ;; for EQ hash tables, and when present the value of
   ;; +MAGIC-HASH-VECTOR-VALUE+ represents EQ-based hashing on the
   ;; respective key.
-  (hash-vector nil :type (or null (simple-array sb!vm:word (*))))
+  (hash-vector nil :type (or null (simple-array sb-vm:word (*))))
   ;; Used for locking GETHASH/(SETF GETHASH)/REMHASH
-  (lock (sb!thread:make-mutex :name "hash-table lock")
-        :type sb!thread:mutex :read-only t)
-  ;; The GC will set this to T if it moves an EQ-based key. This used
-  ;; to be signaled by a bit in the header of the kv vector, but that
-  ;; implementation caused some concurrency issues when we stopped
-  ;; inhibiting GC during hash-table lookup.
-  (needs-rehash-p nil :type (member nil t))
-  ;; Has user requested synchronization?
-  (synchronized-p nil :type (member nil t) :read-only t)
+  (lock (sb-thread:make-mutex :name "hash-table lock")
+        #-c-headers-only :type #-c-headers-only sb-thread:mutex
+        :read-only t)
+  ;; List of values culled out during GC of weak hash table.
+  (culled-values nil :type list)
   ;; For detecting concurrent accesses.
-  #!+sb-hash-table-debug
+  #+sb-hash-table-debug
   (signal-concurrent-access t :type (member nil t))
-  #!+sb-hash-table-debug
+  #+sb-hash-table-debug
   (reading-thread nil)
-  #!+sb-hash-table-debug
+  #+sb-hash-table-debug
   (writing-thread nil))
 
 ;; as explained by pmai on openprojects #lisp IRC 2002-07-30: #x80000000
@@ -103,25 +97,7 @@
 ;; The previous sentence was written when SBCL was 32-bit only. The value
 ;; now depends on the word size. It is propagated to C in genesis because
 ;; the generational garbage collector needs to know it.
-(defconstant +magic-hash-vector-value+ (ash 1 (1- sb!vm:n-word-bits)))
-
-(sb!xc:defmacro with-locked-hash-table ((hash-table) &body body)
-  "Limits concurrent accesses to HASH-TABLE for the duration of BODY.
-If HASH-TABLE is synchronized, BODY will execute with exclusive
-ownership of the table. If HASH-TABLE is not synchronized, BODY will
-execute with other WITH-LOCKED-HASH-TABLE bodies excluded -- exclusion
-of hash-table accesses not surrounded by WITH-LOCKED-HASH-TABLE is
-unspecified."
-  ;; Needless to say, this also excludes some internal bits, but
-  ;; getting there is too much detail when "unspecified" says what
-  ;; is important -- unpredictable, but harmless.
-  `(sb!thread::with-recursive-lock ((hash-table-lock ,hash-table))
-     ,@body))
-
-(sb!xc:defmacro with-locked-system-table ((hash-table) &body body)
-  `(sb!thread::with-recursive-system-lock
-       ((hash-table-lock ,hash-table))
-     ,@body))
+(defconstant +magic-hash-vector-value+ (ash 1 (1- sb-vm:n-word-bits)))
 
 ;;; Return an association list representing the same data as HASH-TABLE.
 (defun %hash-table-alist (hash-table)
