@@ -9,21 +9,18 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;;; miscellaneous global variables
 
 ;;; ANSI: "the floating-point format that is to be used when reading a
 ;;; floating-point number that has no exponent marker or that has e or
 ;;; E for an exponent marker"
-(!defvar *read-default-float-format* 'single-float)
-(declaim (type (member short-float single-float double-float long-float)
-               *read-default-float-format*))
+(defparameter *read-default-float-format* 'single-float)
 
 (defvar *readtable*)
-(declaim (type readtable *readtable*))
 
-(setf (fdocumentation '*readtable* 'variable)
+(setf (documentation '*readtable* 'variable)
       "Variable bound to current readtable.")
 
 ;;; A standard Lisp readtable (once cold-init is through). This is for
@@ -118,7 +115,7 @@
       function
       (cond ((functionp x) x)
             ((null x) ,fallback)
-            (t (sb!c:safe-fdefn-fun x))))))
+            (t (sb-c:safe-fdefn-fun x))))))
 
 ;; Return a function-designator given a character-macro-table entry.
 (defmacro !cmt-entry-to-fun-designator (val)
@@ -133,7 +130,7 @@
 
 ;;; predicates for testing character attributes
 
-#!-sb-fluid
+#-sb-fluid
 (progn
   (declaim (inline whitespace[1]p whitespace[2]p))
   (declaim (inline constituentp terminating-macrop))
@@ -171,9 +168,11 @@
 
 ;;; There are a number of "secondary" attributes which are constant
 ;;; properties of characters (as long as they are constituents).
-
+;;; FIXME: this initform is considered too hairy to assign (a constant array, really?)
+;;; if changed to DEFCONSTANT-EQX, which makes this file unslammable as-is. Oh well.
 (defconstant +constituent-trait-table+
-  #.(let ((a (!make-specialized-array base-char-code-limit '(unsigned-byte 8))))
+  #.(let ((a (sb-xc:make-array base-char-code-limit
+                               :element-type '(unsigned-byte 8))))
       (fill a +char-attr-constituent+)
       (flet ((!set-constituent-trait (char trait)
                (aver (typep char 'base-char))
@@ -191,11 +190,13 @@
         (!set-constituent-trait #\D +char-attr-constituent-expt+)
         (!set-constituent-trait #\S +char-attr-constituent-expt+)
         (!set-constituent-trait #\L +char-attr-constituent-expt+)
+        (!set-constituent-trait #\R +char-attr-constituent-expt+) ; extension
         (!set-constituent-trait #\e +char-attr-constituent-expt+)
         (!set-constituent-trait #\f +char-attr-constituent-expt+)
         (!set-constituent-trait #\d +char-attr-constituent-expt+)
         (!set-constituent-trait #\s +char-attr-constituent-expt+)
         (!set-constituent-trait #\l +char-attr-constituent-expt+)
+        (!set-constituent-trait #\r +char-attr-constituent-expt+) ; extension
         (!set-constituent-trait #\Space +char-attr-invalid+)
         (!set-constituent-trait #\Newline +char-attr-invalid+)
         (dolist (c (list backspace-char-code tab-char-code form-feed-char-code
@@ -567,7 +568,7 @@ standard Lisp readtable when NIL."
 (defun ouch-read-buffer (char buffer)
   ;; When buffer overflow
   (let ((op (token-buf-fill-ptr buffer)))
-    (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+    (declare (optimize (sb-c::insert-array-bounds-checks 0)))
     (when (>= op (length (token-buf-string buffer)))
     ;; an out-of-line call for the uncommon case avoids bloat.
     ;; Size should be doubled.
@@ -590,7 +591,7 @@ standard Lisp readtable when NIL."
 ;; Retun the next character from the buffered token, or NIL.
 (declaim (maybe-inline token-buf-getchar))
 (defun token-buf-getchar (b)
-  (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+  (declare (optimize (sb-c::insert-array-bounds-checks 0)))
   (let ((i (token-buf-cursor (truly-the token-buf b))))
     (and (< i (token-buf-fill-ptr b))
          (prog1 (elt (token-buf-string b) i)
@@ -680,14 +681,14 @@ standard Lisp readtable when NIL."
 ;;; On the other hand, it's probably very very seldom a problem in practice.
 ;;; On the third hand, it might be just as easy to use a hash table,
 ;;; so maybe we should. -- WHN 19991202
-(defvar *sharp-equal* ())
+(defvar *sharp-equal*)
 
 (declaim (ftype (sfunction (t t) (values bit t)) read-maybe-nothing))
 
 ;;; Like READ-PRESERVING-WHITESPACE, but doesn't check the read buffer
 ;;; for being set up properly.
 (defun %read-preserving-whitespace (stream eof-error-p eof-value recursive-p)
-  (declare (optimize (sb!c::check-tag-existence 0)))
+  (declare (optimize (sb-c::check-tag-existence 0)))
   (if recursive-p
       ;; a loop for repeating when a macro returns nothing
       (let* ((tracking-p (form-tracking-stream-p stream))
@@ -794,7 +795,7 @@ standard Lisp readtable when NIL."
         #'(lambda (decoding-error)
             (declare (ignorable decoding-error))
             (style-warn
-             'sb!kernel::character-decoding-error-in-macro-char-comment
+             'sb-kernel::character-decoding-error-in-macro-char-comment
              :position (file-position stream) :stream stream)
             (invoke-restart 'attempt-resync))))
     (let ((stream (in-stream-from-designator stream)))
@@ -912,7 +913,7 @@ standard Lisp readtable when NIL."
   (declare (character closech))
   (macrolet ((scan (read-a-char eofp &optional finish)
                `(loop (let ((char ,read-a-char))
-                        (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+                        (declare (optimize (sb-c::insert-array-bounds-checks 0)))
                         (cond (,eofp (error 'end-of-file :stream stream))
                               ((eql char closech)
                                (return ,finish))
@@ -927,7 +928,7 @@ standard Lisp readtable when NIL."
                                   buf (make-array lim :element-type 'character)))
                           (setq ptr 0))
                         (setf (schar buf ptr) (truly-the character char))
-                        #!+sb-unicode ; BASE-CHAR-P does not exist if not
+                        #+sb-unicode ; BASE-CHAR-P does not exist if not
                         (unless (base-char-p char) (setq only-base-chars nil))
                         (incf ptr)))))
     (let* ((token-buf *read-buffer*)
@@ -1087,7 +1088,7 @@ standard Lisp readtable when NIL."
                      +char-attr-constituent-digit+)
                  +char-attr-constituent-decimal-digit+))
             ((= att +char-attr-invalid+)
-             (simple-reader-error stream "invalid constituent"))
+             (simple-reader-error stream "invalid constituent: ~s" char))
             (t att))))))
 
 ;;;; token fetching
@@ -1097,7 +1098,6 @@ standard Lisp readtable when NIL."
 
 (defvar *read-base* 10
   "the radix that Lisp reads numbers in")
-(declaim (type (integer 2 36) *read-base*))
 
 ;;; Normalize TOKEN-BUF to NFKC, returning a new TOKEN-BUF and the
 ;;; COLON value
@@ -1112,7 +1112,7 @@ standard Lisp readtable when NIL."
     (reset-read-buffer token-buf)
     (macrolet ((clear-str-to-normalize ()
                `(progn
-                  (loop for char across (sb!unicode:normalize-string
+                  (loop for char across (sb-unicode:normalize-string
                                          (subseq str-to-normalize 0 normalize-ptr)
                                          :nfkc) do
                        (ouch-read-buffer char token-buf))
@@ -1144,7 +1144,7 @@ standard Lisp readtable when NIL."
      ((and (zerop (length escapes)) (eq case :upcase))
       (let ((buffer (token-buf-string token-buf)))
         (dotimes (i (token-buf-fill-ptr token-buf))
-          (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+          (declare (optimize (sb-c::insert-array-bounds-checks 0)))
           (setf (schar buffer i) (char-upcase (schar buffer i))))))
      ((eq case :preserve))
      (t
@@ -1155,7 +1155,7 @@ standard Lisp readtable when NIL."
                                   -1 (vector-pop escapes))))
                         ((minusp i))
                       (declare (fixnum i)
-                               (optimize (sb!c::insert-array-bounds-checks 0)))
+                               (optimize (sb-c::insert-array-bounds-checks 0)))
                       (if (< esc i)
                           (let ((ch (schar buffer i)))
                             ,@body)
@@ -1188,20 +1188,27 @@ standard Lisp readtable when NIL."
 (declaim (type (or null package) *reader-package*)
          (always-bound *reader-package*))
 
-(defun reader-find-package (package-designator stream)
+(defun reader-find-package (package-designator stream restarts)
   (if (%instancep package-designator)
       package-designator
-      (let ((package (find-package package-designator)))
-        (cond (package
-               ;; Release the token-buf that was used for the designator
-               (release-token-buf (shiftf (token-buf-next *read-buffer*) nil))
-               package)
-              (t
-               (error 'simple-reader-package-error
-                      :package package-designator
-                      :stream stream
-                      :format-control "Package ~A does not exist."
-                      :format-arguments (list package-designator)))))))
+      (block nil
+        (tagbody retry
+           (let ((package (find-package package-designator)))
+             (cond (package
+                    ;; Release the token-buf that was used for the designator
+                    (release-token-buf (shiftf (token-buf-next *read-buffer*) nil))
+                    (return (values package nil)))
+                   (t
+                    (macrolet ((err ()
+                                 `(error 'simple-reader-package-error
+                                         :package package-designator
+                                         :stream stream
+                                         :format-control "Package ~A does not exist."
+                                         :format-arguments (list package-designator))))
+                      (if restarts
+                          (find-package-restarts (package-designator t)
+                            (err))
+                          (err))))))))))
 
 (defun read-token (stream firstchar)
   "Default readmacro function. Handles numbers, symbols, and SBCL's
@@ -1246,7 +1253,7 @@ extended <package-name>::<form-in-package> syntax."
         (#.+char-attr-package-delimiter+ (go COLON))
         (#.+char-attr-multiple-escape+ (go MULT-ESCAPE))
         (#.+char-attr-invalid+ (simple-reader-error stream
-                                                    "invalid constituent"))
+                                                    "invalid constituent: ~s" char))
         ;; can't have eof, whitespace, or terminating macro as first char!
         (t (go SYMBOL)))
      SIGN ; saw "sign"
@@ -1530,7 +1537,7 @@ extended <package-name>::<form-in-package> syntax."
          (unread-char char stream)
          (if package-designator
              (let* ((*reader-package*
-                     (reader-find-package package-designator stream)))
+                     (reader-find-package package-designator stream nil)))
                (return (read stream t nil t)))
              (simple-reader-error stream
                                   "illegal terminating character after a double-colon: ~S"
@@ -1543,32 +1550,38 @@ extended <package-name>::<form-in-package> syntax."
                               package-designator))
         (t (go SYMBOL)))
       RETURN-SYMBOL
-      (setf buf (normalize-read-buffer buf))
-      (casify-read-buffer buf)
-      (let* ((pkg (if package-designator
-                      (reader-find-package package-designator stream)
-                      (or *reader-package* (sane-package))))
-             (intern-p (or (/= colons 1) (eq pkg *keyword-package*))))
-        (unless intern-p ; Try %FIND-SYMBOL
-          (multiple-value-bind (symbol accessibility)
-              (%find-symbol (token-buf-string buf) (token-buf-fill-ptr buf) pkg)
-            (when (eq accessibility :external) (return symbol))
-            (with-simple-restart (continue "Use symbol anyway.")
-              (error 'simple-reader-package-error
-                     :package pkg
-                     :stream stream
-                     :format-arguments
-                     (list (copy-token-buf-string buf) (package-name pkg))
-                     :format-control
-                     (if accessibility
-                         "The symbol ~S is not external in the ~A package."
-                         "Symbol ~S not found in the ~A package.")))))
-        (return (%intern (token-buf-string buf)
-                         (token-buf-fill-ptr buf)
-                         pkg
-                         (if (token-buf-only-base-chars buf)
-                             (%readtable-symbol-preference rt)
-                             'character))))))))
+        (setf buf (normalize-read-buffer buf))
+        (casify-read-buffer buf)
+        (multiple-value-bind (pkg restart-kind)
+            (if package-designator
+                (reader-find-package package-designator stream t)
+                (or *reader-package* (sane-package)))
+          (if (eq restart-kind :uninterned)
+              (return (make-symbol (copy-token-buf-string buf)))
+              (let* ((intern-p (or (/= colons 1)
+                                   (eq pkg *keyword-package*)
+                                   (eq restart-kind :current))))
+                (unless intern-p        ; Try %FIND-SYMBOL
+                  (multiple-value-bind (symbol accessibility)
+                      (%find-symbol (token-buf-string buf) (token-buf-fill-ptr buf) pkg)
+                    (when (eq accessibility :external) (return symbol))
+                    (with-simple-restart (continue "Use symbol anyway.")
+                      (error 'simple-reader-package-error
+                             :package pkg
+                             :stream stream
+                             :format-arguments
+                             (list (copy-token-buf-string buf) (package-name pkg))
+                             :format-control
+                             (if accessibility
+                                 "The symbol ~S is not external in the ~A package."
+                                 "Symbol ~S not found in the ~A package.")))))
+                (return (%intern (token-buf-string buf)
+                                 (token-buf-fill-ptr buf)
+                                 pkg
+                                 (if (token-buf-only-base-chars buf)
+                                     (%readtable-symbol-preference rt)
+                                     'character)
+                                 nil)))))))))
 
 ;;; For semi-external use: Return 3 values: the token-buf,
 ;;; a flag for whether there was an escape char, and the position of
@@ -1600,7 +1613,7 @@ extended <package-name>::<form-in-package> syntax."
         ((> base 36) a)
       (do ((total (1- base) (+ (* total base) (1- base)))
            (n-digits 0 (1+ n-digits)))
-          ((sb!xc:typep total 'bignum)
+          ((sb-xc:typep total 'bignum)
            (setf (aref a (- base 2)) n-digits))
         ;; empty DO body
         )))
@@ -1613,14 +1626,14 @@ extended <package-name>::<form-in-package> syntax."
            (d (char (write-to-string (1- base) :base base) 0))
            (string (make-string (1+ n-digits) :initial-element d))) ; 1 extra
       (assert (not (typep (parse-integer string :radix base)
-                          `(unsigned-byte ,sb!vm:n-positive-fixnum-bits))))
+                          `(unsigned-byte ,sb-vm:n-positive-fixnum-bits))))
       (assert (typep (parse-integer string :end n-digits :radix base)
-                     `(unsigned-byte ,sb!vm:n-positive-fixnum-bits))))))
+                     `(unsigned-byte ,sb-vm:n-positive-fixnum-bits))))))
 
 (defmacro !setq-optional-leading-sign (sign-flag token-buf rewind)
   ;; guaranteed to have at least one character in buffer at the start
   ;; or immediately following [ESFDL] marker depending on 'rewind' flag.
-  `(locally (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+  `(locally (declare (optimize (sb-c::insert-array-bounds-checks 0)))
      (,(if rewind 'setf 'incf)
        (token-buf-cursor ,token-buf)
        (case (elt (token-buf-string ,token-buf)
@@ -1636,8 +1649,8 @@ extended <package-name>::<form-in-package> syntax."
            (inline token-buf-getchar)) ; makes for smaller code
   (let* ((fixnum-max-digits
           (macrolet ((maxdigits ()
-                       (!coerce-to-specialized  (integer-reader-safe-digits)
-                                                '(unsigned-byte 8))))
+                       (sb-xc:coerce (integer-reader-safe-digits)
+                                     '(vector (unsigned-byte 8)))))
             (aref (maxdigits) (- base 2))))
          (base-power
           (macrolet ((base-powers ()
@@ -1675,16 +1688,16 @@ extended <package-name>::<form-in-package> syntax."
   ;; Use the least positive float, because denormalized exponent
   ;; can be larger than normalized.
   (let* ((max-exponent
-          #!-long-float
-          (+ sb!vm:double-float-digits sb!vm:double-float-bias))
+          #-long-float
+          (+ sb-vm:double-float-digits sb-vm:double-float-bias))
          (number-magnitude (integer-length number))
          (divisor-magnitude (1- (integer-length divisor)))
          (magnitude (- number-magnitude divisor-magnitude)))
     (if (minusp exponent)
         (max exponent (ceiling (- (+ max-exponent magnitude))
-                               #.(floor (log 10 2))))
+                               #.(cl:floor (cl:log 10 2))))
         (min exponent (floor (- max-exponent magnitude)
-                             #.(floor (log 10 2)))))))
+                             #.(cl:floor (cl:log 10 2)))))))
 
 (defun make-float (stream)
   ;; Assume that the contents of *read-buffer* are a legal float, with nothing
@@ -1732,7 +1745,8 @@ extended <package-name>::<form-in-package> syntax."
                                   (#\S 'short-float)
                                   (#\F 'single-float)
                                   (#\D 'double-float)
-                                  (#\L 'long-float)))
+                                  (#\L 'long-float)
+                                  (#\R 'rational)))
                   (exponent (truncate-exponent exponent number divisor))
                   (result (make-float-aux (* (expt 10 exponent) number)
                                           divisor float-format stream)))
@@ -1782,6 +1796,14 @@ extended <package-name>::<form-in-package> syntax."
   (declare (optimize allow-non-returning-tail-call))
   (declare (ignore ignore))
   (if *read-suppress*
+      ;; This seems dubious. For comparison's sake, other implementations
+      ;; will signal an error if the character is not a defined macro.
+      ;; Test case: (read-from-string "#+nope (#!+(or feat) a) b") => B and 25
+      ;; CLISP:
+      ;; *** - READ from #<INPUT STRING-INPUT-STREAM>: After #\# is #\! an undefined dispatch macro character
+      ;; CCL:
+      ;; Error: Reader error on #<STRING-INPUT-STREAM  #x3020004913AD>, near position 10, within "#+nope (#!+(or feat)":
+      ;;        Undefined character #\! in a #\# dispatch macro.
       (values)
       (simple-reader-error stream
                            "no dispatch function defined for ~S"
