@@ -7,10 +7,9 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
-(eval-when (:compile-toplevel)
-  (sb!xc:defmacro %string (x) `(if (stringp ,x) ,x (string ,x))))
+(defmacro %string (x) `(if (stringp ,x) ,x (string ,x)))
 
 (defun string (x)
   "Coerces X into a string. If X is a string, X is returned. If X is a
@@ -35,11 +34,10 @@
 (defun %check-vector-sequence-bounds (vector start end)
   (%check-vector-sequence-bounds vector start end))
 
-(eval-when (:compile-toplevel)
 ;;; WITH-ONE-STRING is used to set up some string hacking things. The
 ;;; keywords are parsed, and the string is hacked into a
 ;;; simple-string.
-(sb!xc:defmacro with-one-string ((string start end) &body forms)
+(defmacro with-one-string ((string start end) &body forms)
   `(let ((,string (%string ,string)))
      (with-array-data ((,string ,string)
                        (,start ,start)
@@ -48,8 +46,8 @@
        ,@forms)))
 ;;; WITH-TWO-STRINGS is used to set up string comparison operations. The
 ;;; keywords are parsed, and the strings are hacked into SIMPLE-STRINGs.
-(sb!xc:defmacro with-two-strings (string1 string2 start1 end1 cum-offset-1
-                                  start2 end2 &rest forms)
+(defmacro with-two-strings (string1 string2 start1 end1 cum-offset-1
+                            start2 end2 &rest forms)
   `(let ((,string1 (%string ,string1))
          (,string2 (%string ,string2)))
      (with-array-data ((,string1 ,string1 :offset-var ,cum-offset-1)
@@ -62,7 +60,7 @@
                          :check-fill-pointer t)
          ,@forms))))
 
-(sb!xc:defmacro with-two-arg-strings (string1 string2 start1 end1 cum-offset-1
+(defmacro with-two-arg-strings (string1 string2 start1 end1 cum-offset-1
                                       start2 end2 &rest forms)
   `(let ((,string1 (%string ,string1))
          (,string2 (%string ,string2)))
@@ -76,8 +74,6 @@
                          :check-fill-pointer t)
          ,@forms))))
 
-) ; EVAL-WHEN
-
 (defun char (string index)
   "Given a string and a non-negative integer index less than the length of
   the string, returns the character object representing the character at
@@ -107,21 +103,21 @@
         (return-from string=* nil))
       ;; Optimizing the non-unicode builds is not terribly important
       ;; because no per-character test for base/UCS4 is needed.
-      #!+sb-unicode
+      #+sb-unicode
       (let* ((widetag1 (%other-pointer-widetag string1))
              (widetag2 (%other-pointer-widetag string2))
              (char-shift
-              #!+(or x86 x86-64)
+              #+(or x86 x86-64)
               ;; The cost of WITH-PINNED-OBJECTS is near nothing on x86,
               ;; and memcmp() is much faster except below a cutoff point.
               ;; The threshold is higher on x86-32 because the overhead
               ;; of a foreign call is higher due to FPU stack save/restore.
               (if (and (= widetag1 widetag2)
-                       (>= len #!+x86 16
-                               #!+x86-64 8))
+                       (>= len #+x86 16
+                               #+x86-64 8))
                   (case widetag1
-                    (#.sb!vm:simple-base-string-widetag 0)
-                    (#.sb!vm:simple-character-string-widetag 2)))))
+                    (#.sb-vm:simple-base-string-widetag 0)
+                    (#.sb-vm:simple-character-string-widetag 2)))))
         (when char-shift
           (return-from string=*
             ;; Efficiently compute byte indices. Derive-type on ASH isn't
@@ -133,9 +129,9 @@
                          `(sap+ (vector-sap (truly-the string ,string))
                                 (scale ,start)))
                        (scale (index)
-                         `(truly-the sb!vm:signed-word
+                         `(truly-the sb-vm:signed-word
                            (ash (truly-the index ,index) char-shift))))
-              (declare (optimize (sb!c:alien-funcall-saves-fp-and-pc 0)))
+              (declare (optimize (sb-c:alien-funcall-saves-fp-and-pc 0)))
               (with-pinned-objects (string1 string2)
                 (zerop (alien-funcall
                         (extern-alien "memcmp"
@@ -147,7 +143,7 @@
                `(return-from string=*
                   (let ((string1 (truly-the (simple-array ,type1 1) string1))
                         (string2 (truly-the (simple-array ,type2 1) string2)))
-                    (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+                    (declare (optimize (sb-c::insert-array-bounds-checks 0)))
                     (do ((index1 start1 (1+ index1))
                          (index2 start2 (1+ index2)))
                         ((>= index1 end1) t)
@@ -163,17 +159,17 @@
           ;; On non-x86, Lisp code is used always because I did not profile
           ;; memcmp(), and this code is at least as good as %SP-STRING-COMPARE.
           ;; Also, (ARRAY NIL) always punts.
-          (cond #!-x86-64
+          (cond #-x86-64
                 ((= widetag1 widetag2)
                  (case widetag1
-                   (#.sb!vm:simple-base-string-widetag
+                   (#.sb-vm:simple-base-string-widetag
                     (char-loop base-char base-char))
-                   (#.sb!vm:simple-character-string-widetag
+                   (#.sb-vm:simple-character-string-widetag
                     (char-loop character character))))
-                ((or (and (= widetag1 sb!vm:simple-character-string-widetag)
-                          (= widetag2 sb!vm:simple-base-string-widetag))
-                     (and (= widetag2 sb!vm:simple-character-string-widetag)
-                          (= widetag1 sb!vm:simple-base-string-widetag)
+                ((or (and (= widetag1 sb-vm:simple-character-string-widetag)
+                          (= widetag2 sb-vm:simple-base-string-widetag))
+                     (and (= widetag2 sb-vm:simple-character-string-widetag)
+                          (= widetag1 sb-vm:simple-base-string-widetag)
                           (progn (rotatef start1 start2)
                                  (rotatef end1 end2)
                                  (rotatef string1 string2)
@@ -187,11 +183,9 @@
                                           string2 start2 end2)))
       (if comparison (- (the fixnum comparison) offset1)))))
 
-(eval-when (:compile-toplevel :execute)
-
 ;;; LESSP is true if the desired expansion is for STRING<* or STRING<=*.
 ;;; EQUALP is true if the desired expansion is for STRING<=* or STRING>=*.
-(sb!xc:defmacro string<>=*-body (lessp equalp)
+(defmacro string<>=*-body (lessp equalp)
   (let ((offset1 (gensym)))
     `(with-two-strings string1 string2 start1 end1 ,offset1 start2 end2
        (let ((index (%sp-string-compare string1 start1 end1
@@ -212,7 +206,6 @@
                     (- (the fixnum index) ,offset1))
                    (t nil))
              ,(if equalp `(- (the fixnum end1) ,offset1) nil))))))
-) ; EVAL-WHEN
 
 (defun string<* (string1 string2 start1 end1 start2 end2)
   (declare (fixnum start1 start2))
@@ -292,33 +285,29 @@
 (defun two-arg-string/= (string1 string2)
   (string/=* string1 string2 0 nil 0 nil))
 
-(eval-when (:compile-toplevel :execute)
-
 ;;; STRING-NOT-EQUAL-LOOP is used to generate character comparison loops for
 ;;; STRING-EQUAL and STRING-NOT-EQUAL.
-(sb!xc:defmacro string-not-equal-loop (end
-                                       end-value
+(defmacro string-not-equal-loop (end end-value
                                        &optional (abort-value nil abortp))
   (declare (fixnum end))
   (let ((end-test (if (= end 1)
                       `(= index1 (the fixnum end1))
                       `(= index2 (the fixnum end2)))))
-    `(locally (declare (inline two-arg-char-equal))
-       (do ((index1 start1 (1+ index1))
-            (index2 start2 (1+ index2)))
-           (,(if abortp
-                 end-test
-                 `(or ,end-test
-                      (not (char-equal (schar string1 index1)
-                                       (schar string2 index2)))))
-            ,end-value)
-         (declare (fixnum index1 index2))
-         ,@(if abortp
-               `((if (not (char-equal (schar string1 index1)
-                                      (schar string2 index2)))
-                     (return ,abort-value))))))))
-
-) ; EVAL-WHEN
+    `(do ((index1 start1 (1+ index1))
+         (index2 start2 (1+ index2)))
+        (,(if abortp
+              end-test
+              `(or ,end-test
+                   (not (two-arg-char-equal-inline
+                         (schar string1 index1)
+                         (schar string2 index2)))))
+         ,end-value)
+      (declare (fixnum index1 index2))
+      ,@(if abortp
+            `((if (not (two-arg-char-equal-inline
+                        (schar string1 index1)
+                        (schar string2 index2)))
+                  (return ,abort-value)))))))
 
 (defun string-equal (string1 string2 &key (start1 0) end1 (start2 0) end2)
   "Given two strings (string1 and string2), and optional integers start1,
@@ -368,7 +357,7 @@
             (t
              (string-not-equal-loop 2 (- index1 offset1)))))))
 
-(eval-when (:compile-toplevel :execute)
+(eval-when (:compile-toplevel :load-toplevel :execute)
 
 ;;; STRING-LESS-GREATER-EQUAL-TESTS returns a test on the lengths of string1
 ;;; and string2 and a test on the current characters from string1 and string2
@@ -385,30 +374,28 @@
           (values '>= `(not (char-lessp char1 char2)))
           ;; STRING-GREATERP
           (values '> `(char-greaterp char1 char2)))))
+) ; EVAL-WHEN
 
-(sb!xc:defmacro string-less-greater-equal (lessp equalp)
+(defmacro string-less-greater-equal (lessp equalp)
   (multiple-value-bind (length-test character-test)
       (string-less-greater-equal-tests lessp equalp)
-    `(locally (declare (inline two-arg-char-equal))
-       (with-two-strings string1 string2 start1 end1 offset1 start2 end2
-         (let ((slen1 (- (the fixnum end1) start1))
-               (slen2 (- (the fixnum end2) start2)))
-           (declare (fixnum slen1 slen2))
-           (do ((index1 start1 (1+ index1))
-                (index2 start2 (1+ index2))
-                (char1)
-                (char2))
-               ((or (= index1 (the fixnum end1)) (= index2 (the fixnum end2)))
-                (if (,length-test slen1 slen2) (- index1 offset1)))
-             (declare (fixnum index1 index2))
-             (setq char1 (schar string1 index1))
-             (setq char2 (schar string2 index2))
-             (if (not (char-equal char1 char2))
-                 (if ,character-test
-                     (return (- index1 offset1))
-                     (return ())))))))))
-
-) ; EVAL-WHEN
+    `(with-two-strings string1 string2 start1 end1 offset1 start2 end2
+       (let ((slen1 (- (the fixnum end1) start1))
+             (slen2 (- (the fixnum end2) start2)))
+         (declare (fixnum slen1 slen2))
+         (do ((index1 start1 (1+ index1))
+              (index2 start2 (1+ index2))
+              (char1)
+              (char2))
+             ((or (= index1 (the fixnum end1)) (= index2 (the fixnum end2)))
+              (if (,length-test slen1 slen2) (- index1 offset1)))
+           (declare (fixnum index1 index2))
+           (setq char1 (schar string1 index1))
+           (setq char2 (schar string2 index2))
+           (if (not (two-arg-char-equal-inline char1 char2))
+               (if ,character-test
+                   (return (- index1 offset1))
+                   (return ()))))))))
 
 (defun string-lessp* (string1 string2 start1 end1 start2 end2)
   (declare (fixnum start1 start2))
@@ -598,5 +585,5 @@ new string COUNT long filled with the fill character."
   ;;  #b10_ ; literal compiled to core
   (set-header-data (the (simple-array * 1) vector)
                    (if always-shareable
-                       sb!vm:+vector-shareable+
-                       sb!vm:+vector-shareable-nonstd+)))
+                       sb-vm:+vector-shareable+
+                       sb-vm:+vector-shareable-nonstd+)))
