@@ -9,59 +9,69 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;; We compile some trivial character operations via inline expansion.
-#!-sb-fluid
+#-sb-fluid
 (declaim (inline standard-char-p graphic-char-p alpha-char-p
                  alphanumericp))
 (declaim (maybe-inline upper-case-p lower-case-p both-case-p
-                       digit-char-p two-arg-char-equal))
+                       digit-char-p))
 
 (deftype char-code ()
-  `(integer 0 (,sb!xc:char-code-limit)))
+  `(integer 0 (,sb-xc:char-code-limit)))
 
-(defglobal **unicode-character-name-huffman-tree** ())
+(define-load-time-global **unicode-character-name-huffman-tree** ())
 
 (declaim (inline pack-3-codepoints))
 (defun pack-3-codepoints (first &optional (second 0) (third 0))
   (declare (type (unsigned-byte 21) first second third))
-  (sb!c::mask-signed-field 63 (logior first (ash second 21) (ash third 42))))
+  (sb-c::mask-signed-field 63 (logior first (ash second 21) (ash third 42))))
+
+(defun unpack-3-codepoints (codepoints)
+  (declare (type (signed-byte 63) codepoints))
+  (cond ((< codepoints (ash 1 21))
+         (list (code-char codepoints)))
+        ((< codepoints (ash 1 (* 21 2)))
+         (list (code-char (ldb (byte 21 0) codepoints))
+               (code-char (ldb (byte 21 21) codepoints))))
+        (t
+         (list (code-char (ldb (byte 21 0) codepoints))
+               (code-char (ldb (byte 21 21) codepoints))
+               (code-char (ldb (byte 21 (* 21 2)) codepoints))))))
 
 (macrolet ((frob ()
              (flet ((coerce-it (array)
-                      (!coerce-to-specialized array '(unsigned-byte 8)))
+                      (sb-xc:coerce array '(simple-array (unsigned-byte 8) 1)))
                     (file (name type)
-                      ;; FIXME: this fails on a certain build system,
-                      ;; as does the more obvious :directory '(:relative "output").
-                      ;; Why???
-                      (make-pathname :directory (pathname-directory (merge-pathnames "output/"))
-                                     :name name :type type))
+                      (let ((dir (sb-cold:prepend-genfile-path "output/")))
+                        (make-pathname :directory (pathname-directory (merge-pathnames dir))
+                                       :name name :type type)))
                     (read-ub8-vector (pathname)
                       (with-open-file (stream pathname
                                               :element-type '(unsigned-byte 8))
                         (let* ((length (file-length stream))
-                               (array (make-array
+                               (array (sb-xc:make-array
                                        length :element-type '(unsigned-byte 8))))
                           (read-sequence array stream)
                           array)))
                     (init-global (name type &optional length)
                       `(progn
-                         (defglobal ,name
+                         (define-load-time-global ,name
                              ,(if (eql type 'hash-table)
                                   `(make-hash-table)
                                   `(make-array ,length :element-type ',type)))
                          (declaim (type ,(if (eql type 'hash-table)
                                              'hash-table
                                              `(simple-array ,type (,length))) ,name)))))
-               (let ((misc-database (coerce-it (read-ub8-vector (file "ucdmisc" "dat"))))
-                     (ucd-high-pages (coerce-it (read-ub8-vector (file "ucdhigh" "dat"))))
-                     (ucd-low-pages (coerce-it (read-ub8-vector (file "ucdlow" "dat"))))
-                     (decompositions (coerce-it (read-ub8-vector (file "decomp" "dat"))))
-                     (primary-compositions (coerce-it (read-ub8-vector (file "comp" "dat"))))
-                     (case-data (coerce-it (read-ub8-vector (file "case" "dat"))))
-                     (case-pages (coerce-it (read-ub8-vector (file "casepages" "dat"))))
-                     (collations (coerce-it (read-ub8-vector (file "collation" "dat")))))
+               (let ((misc-database (read-ub8-vector (file "ucdmisc" "dat")))
+                     (ucd-high-pages (read-ub8-vector (file "ucdhigh" "dat")))
+                     (ucd-low-pages (read-ub8-vector (file "ucdlow" "dat")))
+                     (decompositions (read-ub8-vector (file "decomp" "dat")))
+                     (primary-compositions (read-ub8-vector (file "comp" "dat")))
+                     (case-data (read-ub8-vector (file "case" "dat")))
+                     (case-pages (read-ub8-vector (file "casepages" "dat")))
+                     (collations (read-ub8-vector (file "collation" "dat"))))
                  `(progn
                     ;; KLUDGE: All temporary values, fixed up in cold-load
                     ,(init-global '**character-misc-database** '(unsigned-byte 8)
@@ -88,7 +98,7 @@
                                (let ((new-array
                                        (make-array
                                         (/ (length raw-bytes) n)
-                                        :element-type (list 'unsigned-byte (* 8 n)))))
+                                        :element-type `(unsigned-byte ,(* 8 n)))))
                                  (loop for i from 0 below (length raw-bytes) by n
                                        for element = 0 do
                                        (loop for offset from 0 below n do
@@ -109,8 +119,9 @@
                               (make-ubn-vector ,decompositions 3))
 
                         (setf **character-primary-compositions**
-                              (let ((table (make-hash-table))
-                                    (info (make-ubn-vector ,primary-compositions 3)))
+                              (let* ((info (make-ubn-vector ,primary-compositions 3))
+                                     (table (make-hash-table #+64-bit :test #+64-bit #'eq
+                                                             :size (/ (length info) 3))))
                                 (dotimes (i (/ (length info) 3))
                                   (setf (gethash (dpb (aref info (* 3 i)) (byte 21 21)
                                                       (aref info (1+ (* 3 i))))
@@ -151,9 +162,13 @@
                                   for page = (aref **character-case-pages** (ash key -6))
                                   for i = (+ (ash page 6) (ldb (byte 6 0) key))
                                   do
-                                  (setf (aref unicode-table i) (cons upper lower))
+                                  (setf (aref unicode-table i)
+                                        (if (or (consp upper)
+                                                (consp lower))
+                                            (cons upper lower)
+                                            (dpb upper (byte 21 21) lower)))
                                   when
-                                  (flet (#!+sb-unicode
+                                  (flet (#+sb-unicode
                                          (both-case-p (code)
                                            (logbitp 7 (aref **character-misc-database**
                                                             (+ 5 (misc-index (code-char code)))))))
@@ -161,9 +176,9 @@
                                          (atom lower)
                                          ;; Some characters are only equal under unicode rules,
                                          ;; e.g. #\MICRO_SIGN and #\GREEK_CAPITAL_LETTER_MU
-                                         #!+sb-unicode
+                                         #+sb-unicode
                                          (both-case-p lower)
-                                         #!+sb-unicode
+                                         #+sb-unicode
                                          (both-case-p upper)))
                                   do
                                   (setf (aref table (* i 2)) lower
@@ -172,7 +187,8 @@
                           (setf **character-cases** table))
 
                         (setf **character-collations**
-                              (let* ((table (make-hash-table))
+                              (let* ((table (make-hash-table :size 27978
+                                                             #+64-bit :test #+64-bit #'eq))
                                      (index 0)
                                      (info (make-ubn-vector ,collations 4))
                                      (len (length info)))
@@ -180,22 +196,20 @@
                                       (let* ((entry-head (aref info index))
                                              (cp-length (ldb (byte 4 28) entry-head))
                                              (key-length (ldb (byte 5 23) entry-head))
-                                             (key (make-array
-                                                   key-length
-                                                   :element-type '(unsigned-byte 32)))
+                                             (key 0)
                                              (codepoints nil))
-                                        (assert (and (/= cp-length 0) (/= key-length 0)))
+                                        (aver (and (/= cp-length 0) (/= key-length 0)))
                                         (loop repeat cp-length do
                                               (push (dpb 0 (byte 10 22) (aref info index))
                                                     codepoints)
                                               (incf index))
                                         (setf codepoints (nreverse codepoints))
                                         (dotimes (i key-length)
-                                          (setf (aref key i) (aref info index))
+                                          (setf (ldb (byte 32 (* i 32)) key) (aref info index))
                                           (incf index))
-                                        (setf (gethash
-                                               (apply #'pack-3-codepoints codepoints)
-                                               table) (logically-readonlyize key))))
+                                        (setf (gethash (apply #'pack-3-codepoints codepoints) table)
+                                              key)))
+                                (assert (= (hash-table-count table) 27978))
                                 table))))
 
                     ,(with-open-file
@@ -284,7 +298,7 @@
   (frob))
 #+sb-xc-host (!character-name-database-cold-init)
 
-(defglobal *base-char-name-alist*
+(define-load-time-global *base-char-name-alist*
   ;; Note: The *** markers here indicate character names which are
   ;; required by the ANSI specification of #'CHAR-NAME. For the others,
   ;; we prefer the ASCII standard name.
@@ -568,7 +582,7 @@ argument is an alphabetic character, A-Z or a-z; otherwise NIL."
         (page-var (gensym "PAGE")))
     `(block nil
        (locally
-           (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+           (declare (optimize (sb-c::insert-array-bounds-checks 0)))
          (let ((,code-var (char-code ,char)))
            (let* ((,shifted-var (ash ,code-var -6))
                   (,page-var (if (>= ,shifted-var (length **character-case-pages**))
@@ -611,7 +625,7 @@ lowercase eszet (U+DF)."
     (let ((code (aref cases (1+ index))))
       (if (zerop code)
           char
-          (code-char code)))))
+          (code-char (truly-the char-code code))))))
 
 (defun char-downcase (char)
   "Return CHAR converted to lower-case if that is possible."
@@ -620,7 +634,7 @@ lowercase eszet (U+DF)."
     (let ((code (aref cases index)))
       (if (zerop code)
           char
-          (code-char code)))))
+          (code-char (truly-the char-code code))))))
 
 (defun alphanumericp (char)
   "Given a character-object argument, ALPHANUMERICP returns T if the argument
@@ -664,7 +678,8 @@ is either numeric or alphabetic."
               code
               down-code)))))
 
-(defun two-arg-char-equal (c1 c2)
+(declaim (inline two-arg-char-equal-inline))
+(defun two-arg-char-equal-inline (c1 c2)
   (flet ((base-char-equal-p ()
            (let* ((code1 (char-code c1))
                   (code2 (char-code c2))
@@ -676,30 +691,28 @@ is either numeric or alphabetic."
                      (and (< 463 sum 477))))))))
     (declare (inline base-char-equal-p))
     (cond ((eq c1 c2))
-          #!-sb-unicode
+          #-sb-unicode
           (t
            (base-char-equal-p))
-          #!+sb-unicode
+          #+sb-unicode
           ((base-char-p c1)
            (and (base-char-p c2)
                 (base-char-equal-p)))
-          #!+sb-unicode
+          #+sb-unicode
           ((base-char-p c2)
            nil)
-          #!+sb-unicode
+          #+sb-unicode
           (t
            (with-case-info (c1 index cases)
              (or (= (aref cases index) (char-code c2)) ;; lower case
                  (= (aref cases (1+ index)) (char-code c2))))))))
 
-(defun char-equal-constant (x char reverse-case-char)
-  (declare (type character x) (explicit-check))
-  (or (eq char x)
-      (eq reverse-case-char x)))
+;;; There are transforms on two-arg-char-equal, don't make it inlinable itself.
+(defun two-arg-char-equal (c1 c2)
+  (two-arg-char-equal-inline c1 c2))
 
 (defun two-arg-char-not-equal (c1 c2)
-  (declare (inline two-arg-char-equal))
-  (not (two-arg-char-equal c1 c2)))
+  (not (two-arg-char-equal-inline c1 c2)))
 
 (macrolet ((def (name test doc)
              `(defun ,name (character &rest more-characters)
