@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!ARM64-ASM")
+(in-package "SB-ARM64-ASM")
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
@@ -19,15 +19,14 @@
             negative-add-sub-immediate-p
             encode-logical-immediate fixnum-encode-logical-immediate
             ldr-str-offset-encodable ldp-stp-offset-p
-            bic-mask extend lsl lsr asr ror @) "SB!VM")
+            bic-mask extend lsl lsr asr ror @) "SB-VM")
   ;; Imports from SB-VM into this package
-  (import '(sb!vm::*register-names*
-            sb!vm::add-sub-immediate
-            sb!vm::32-bit-reg sb!vm::single-reg sb!vm::double-reg
-            sb!vm::complex-single-reg sb!vm::complex-double-reg
-            sb!vm::tmp-tn sb!vm::zr-tn sb!vm::nsp-offset)))
+  (import '(sb-vm::*register-names*
+            sb-vm::add-sub-immediate
+            sb-vm::32-bit-reg sb-vm::single-reg sb-vm::double-reg
+            sb-vm::complex-single-reg sb-vm::complex-double-reg
+            sb-vm::tmp-tn sb-vm::zr-tn sb-vm::nsp-offset)))
 
-(setf *disassem-inst-alignment-bytes* 4)
 
 
 (defconstant-eqx +conditions+
@@ -48,23 +47,18 @@
     (:al . 14))
   #'equal)
 
-(defconstant-eqx sb!vm::+condition-name-vec+
+(defconstant-eqx sb-vm::+condition-name-vec+
   #.(let ((vec (make-array 16 :initial-element nil)))
       (dolist (cond +conditions+ vec)
         (when (null (aref vec (cdr cond)))
           (setf (aref vec (cdr cond)) (car cond)))))
   #'equalp)
 
-;;; Set assembler parameters. (In CMU CL, this was done with
-;;; a call to a macro DEF-ASSEMBLER-PARAMS.)
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (setf *assem-scheduler-p* nil))
-
 (defun conditional-opcode (condition)
   (cdr (assoc condition +conditions+ :test #'eq)))
 
 (defun invert-condition (condition)
-  (aref sb!vm::+condition-name-vec+
+  (aref sb-vm::+condition-name-vec+
         (logxor 1 (conditional-opcode condition))))
 
 ;;;; disassembler field definitions
@@ -135,11 +129,11 @@
 
 (defun register-p (thing)
   (and (tn-p thing)
-       (eq (sb-name (sc-sb (tn-sc thing))) 'sb!vm::registers)))
+       (eq (sb-name (sc-sb (tn-sc thing))) 'sb-vm::registers)))
 
 (defun fp-register-p (thing)
   (and (tn-p thing)
-       (eq (sb-name (sc-sb (tn-sc thing))) 'sb!vm::float-registers)))
+       (eq (sb-name (sc-sb (tn-sc thing))) 'sb-vm::float-registers)))
 
 (defun reg-size (tn)
   (if (sc-is tn 32-bit-reg)
@@ -375,7 +369,7 @@
                                    (car pieces)))
                   forms)))
         `(defun ,(symbolicate "EMIT-" name) (segment ,@(arg-names))
-           (declare (type sb!assem:segment segment) ,@(arg-types))
+           (declare (type sb-assem:segment segment) ,@(arg-types))
            ,@(ecase *backend-byte-order*
                (:little-endian (nreverse forms))
                (:big-endian forms))
@@ -968,21 +962,14 @@
   (imm :field (byte 16 5) :type 'unsigned-immediate)
   (rd :field (byte 5 0) :type 'reg))
 
-(defmacro process-null-sc (reg)
-  `(setf ,reg (if (and (tn-p ,reg)
-                       (eq 'null (sc-name (tn-sc ,reg))))
-                  sb!vm::null-tn
-                  ,reg)))
-
 (define-instruction-macro mov-sp (rd rm)
   `(inst add ,rd ,rm 0))
 
 (define-instruction-macro mov (rd rm)
   `(let ((rd ,rd)
          (rm ,rm))
-     (process-null-sc rm)
      (if (integerp rm)
-         (sb!vm::load-immediate-word rd rm)
+         (sb-vm::load-immediate-word rd rm)
          (inst orr rd zr-tn rm))))
 
 (define-instruction movn (segment rd imm &optional (shift 0))
@@ -1309,7 +1296,6 @@
              (typep qout '(unsigned-byte 12))))))
 
 (defun emit-load-store (size opc segment dst address)
-  (process-null-sc dst)
   (let* ((base (memory-operand-base address))
          (offset (memory-operand-offset address))
          (mode (memory-operand-mode address))
@@ -1489,7 +1475,7 @@
                       (fp-reg-type rt1)))))
     (when fp
       (setf size (+ opc 2)))
-    (assert (not (ldb-test (byte size 0) offset)))
+    (aver (not (ldb-test (byte size 0) offset)))
     (emit-ldr-str-pair segment opc v
                        (ecase mode
                          (:post-index #b01)
@@ -1613,13 +1599,13 @@
          (t
           (emit-back-patch segment 4
                            (cond (label
-                                  (assert (label-p label))
+                                  (aver (label-p label))
                                   (lambda (segment posn)
                                     (emit-cond-branch segment
                                                       (ash (- (label-position label) posn) -2)
                                                       (conditional-opcode cond-or-label))))
                                  (t
-                                  (assert (label-p cond-or-label))
+                                  (aver (label-p cond-or-label))
                                   (lambda (segment posn)
                                     (emit-uncond-branch segment
                                                         0
@@ -1664,9 +1650,9 @@
   (:emitter
    (emit-uncond-branch-reg segment 1 (tn-offset register))))
 
-(define-instruction ret (segment &optional (register sb!vm::lr-tn))
+(define-instruction ret (segment &optional (register sb-vm::lr-tn))
   (:printer uncond-branch-reg ((op #b10)))
-  (:printer uncond-branch-reg ((op #b10) (rn sb!vm::lr-offset))
+  (:printer uncond-branch-reg ((op #b10) (rn sb-vm::lr-offset))
             '(:name))
   (:emitter
    (emit-uncond-branch-reg segment #b10 (tn-offset register))))
@@ -1691,7 +1677,7 @@
 (define-instruction cbz (segment rt label)
   (:printer compare-branch-imm ((op 0)))
   (:emitter
-   (assert (label-p label))
+   (aver (label-p label))
    (emit-back-patch segment 4
                     (lambda (segment posn)
                       (emit-compare-branch-imm segment
@@ -1703,7 +1689,7 @@
 (define-instruction cbnz (segment rt label)
   (:printer compare-branch-imm ((op 1)))
   (:emitter
-   (assert (label-p label))
+   (aver (label-p label))
    (emit-back-patch segment 4
                     (lambda (segment posn)
                       (emit-compare-branch-imm segment
@@ -1731,7 +1717,7 @@
 (define-instruction tbz (segment rt bit label)
   (:printer test-branch-imm ((op 0)))
   (:emitter
-   (assert (label-p label))
+   (aver (label-p label))
    (check-type bit (integer 0 63))
    (emit-back-patch segment 4
                     (lambda (segment posn)
@@ -1745,7 +1731,7 @@
 (define-instruction tbnz (segment rt bit label)
   (:printer test-branch-imm ((op 1)))
   (:emitter
-   (assert (label-p label))
+   (aver (label-p label))
    (check-type bit (integer 0 63))
    (emit-back-patch segment 4
                     (lambda (segment posn)
@@ -1798,8 +1784,8 @@
   (rd :field (byte 5 0) :type 'x-reg))
 
 (defun emit-pc-relative-inst (op segment rd label &optional (offset 0))
-  (assert (label-p label))
-  (assert (register-p rd))
+  (aver (label-p label))
+  (aver (register-p rd))
   (emit-back-patch segment 4
                    (lambda (segment posn)
                      (let ((offset (+ (- (label-position label) posn)
@@ -1965,9 +1951,9 @@
   `(define-instruction ,name (segment rn rm)
      (:printer fp-compare ((op ,op)))
      (:printer fp-compare ((op ,op) (z 1) (type 0))
-               '(:name :tab rn ", " 0s0))
+               '(:name :tab rn ", " $0f0))
      (:printer fp-compare ((op ,op) (z 1) (type 1))
-               '(:name :tab rn ", " 0d0))
+               '(:name :tab rn ", " $0d0))
      (:emitter
       (assert (or (eql rm 0)
                   (eq (tn-sc rn)
@@ -2276,7 +2262,8 @@
                                  (ldb (byte 2 0) delta)
                                  (ldb (byte 19 2) delta)
                                  (tn-offset dest))))
-           (multi-instruction-maybe-shrink (segment posn magic-value)
+           (multi-instruction-maybe-shrink (segment chooser posn magic-value)
+             (declare (ignore chooser))
              (when (typep (funcall compute-delta posn magic-value)
                           '(signed-byte 21))
                (emit-back-patch segment 4
@@ -2289,6 +2276,7 @@
 
 (define-instruction compute-code (segment code lip object-label)
   (:vop-var vop)
+  (:declare (ignore object-label))
   (:emitter
    (emit-compute segment vop code lip
                  (lambda (position &optional magic-value)
@@ -2338,7 +2326,8 @@
                                 (ldb (byte 19 0)
                                      (ash (compute-delta position) -2))
                                 (tn-offset dest)))
-            (multi-instruction-maybe-shrink (segment posn magic-value)
+            (multi-instruction-maybe-shrink (segment chooser posn magic-value)
+              (declare (ignore chooser))
               (let ((delta (compute-delta posn magic-value)))
                 (when (typep delta '(signed-byte 21))
                   (emit-back-patch segment 4
@@ -2509,7 +2498,7 @@
          (aver (integerp value))
          (cons type value))
         (:base-char
-         #!+sb-unicode (aver (typep value 'base-char))
+         #+sb-unicode (aver (typep value 'base-char))
          (cons :byte (char-code value)))
         (:character
          (aver (characterp value))
@@ -2521,8 +2510,7 @@
         (:double-float
          (aver (typep value 'double-float))
          (cons (if alignedp :oword :qword)
-               (ldb (byte 64 0) (logior (ash (double-float-high-bits value) 32)
-                                        (double-float-low-bits value)))))
+               (ldb (byte 64 0) (double-float-bits value))))
         (:complex-single-float
          (aver (typep value '(complex single-float)))
          (cons (if alignedp :oword :qword)
@@ -2533,12 +2521,8 @@
         (:complex-double-float
          (aver (typep value '(complex double-float)))
          (cons :oword
-               (logior (ash (double-float-high-bits (imagpart value)) 96)
-                       (ash (double-float-low-bits (imagpart value)) 64)
-                       (ash (ldb (byte 32 0)
-                                 (double-float-high-bits (realpart value)))
-                            32)
-                       (double-float-low-bits (realpart value)))))
+               (logior (ash (ldb (byte 64 0) (double-float-bits (imagpart value))) 64)
+                       (ldb (byte 64 0) (double-float-bits (realpart value))))))
         (:fixup
          (cons :fixup value))))))
 
@@ -2561,16 +2545,20 @@
   (stable-sort constants #'> :key (lambda (constant)
                                     (size-nbyte (caar constant)))))
 
-(defun emit-inline-constant (constant label)
+(defun emit-inline-constant (section constant label)
   (let* ((type (car constant))
          (size (size-nbyte type)))
-    (emit-alignment (integer-length (1- size)))
-    (emit-label label)
-    (let ((val (cdr constant)))
-      (case type
-        (:fixup
-         (inst word (apply #'make-fixup val)))
-        (t
-         (loop repeat size
-               do (inst byte (ldb (byte 8 0) val))
-                  (setf val (ash val -8))))))))
+    (emit section
+          `(.align ,(integer-length (1- size)))
+          label
+          (let ((val (cdr constant)))
+            (case type
+              (:fixup
+               ;; Use the WORD emitter which knows how to emit fixups
+               `(|word| ,(apply #'make-fixup val)))
+              (t
+               ;; Could add pseudo-ops for .WORD, .INT, .QUAD, .OCTA just like gcc has.
+               ;; But it works fine to emit as a sequence of bytes
+               `(.byte ,@(loop repeat size
+                               collect (prog1 (ldb (byte 8 0) val)
+                                         (setf val (ash val -8)))))))))))
