@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 
 ;;;; register specs
@@ -51,9 +51,9 @@
   (defreg r8 18)
   (defreg r9 19)
 
-  #!+sb-thread
+  #+sb-thread
   (defreg thread 20)
-  #!-sb-thread
+  #-sb-thread
   (defreg r10 20)
 
   (defreg lexenv 21)
@@ -74,7 +74,7 @@
       null cfp nsp lr code)
 
   (defregset descriptor-regs
-      r0 r1 r2 r3 r4 r5 r6 r7 r8 r9 #!-sb-thread r10 lexenv)
+      r0 r1 r2 r3 r4 r5 r6 r7 r8 r9 #-sb-thread r10 lexenv)
 
   (defregset non-descriptor-regs
       nl0 nl1 nl2 nl3 nl4 nl5 nl6 nl7 nl8 nl9 nargs nfp ocfp)
@@ -82,7 +82,7 @@
   ;; OAOOM: Same as runtime/arm64-lispregs.h
   (defregset boxed-regs
       r0 r1 r2 r3 r4 r5 r6
-      r7 r8 r9 #!-sb-thread r10 #!+sb-thread thread lexenv code)
+      r7 r8 r9 #-sb-thread r10 #+sb-thread thread lexenv code)
 
   ;; registers used to pass arguments
   ;;
@@ -108,9 +108,6 @@
   ;; Non-immediate contstants in the constant pool
   (constant constant)
 
-  ;; NULL is in a register.
-  (null immediate-constant)
-
   ;; Anything else that can be an immediate.
   (immediate immediate-constant)
 
@@ -137,7 +134,7 @@
   ;; Pointer descriptor objects.  Must be seen by GC.
   (descriptor-reg registers
                   :locations #.descriptor-regs
-                  :constant-scs (constant null immediate)
+                  :constant-scs (constant immediate)
                   :save-p t
                   :alternate-scs (control-stack))
 
@@ -243,7 +240,7 @@
   (defregtn cfp any-reg)
   (defregtn csp any-reg)
   (defregtn lr interior-reg)
-  #!+sb-thread
+  #+sb-thread
   (defregtn thread interior-reg))
 
 ;;; If VALUE can be represented as an immediate constant, then return the
@@ -251,8 +248,8 @@
 (defun immediate-constant-sc (value)
   (typecase value
     (null
-     null-sc-number)
-    ((or (integer #.sb!xc:most-negative-fixnum #.sb!xc:most-positive-fixnum)
+     (values descriptor-reg-sc-number null-offset))
+    ((or (integer #.sb-xc:most-negative-fixnum #.sb-xc:most-positive-fixnum)
          character)
      immediate-sc-number)
     (symbol
@@ -261,8 +258,7 @@
          nil))))
 
 (defun boxed-immediate-sc-p (sc)
-  (or (eql sc null-sc-number)
-      (eql sc immediate-sc-number)))
+  (eql sc immediate-sc-number))
 
 ;;;; function call parameters
 
@@ -313,10 +309,10 @@
 
 (defun combination-implementation-style (node)
   (flet ((valid-funtype (args result)
-           (sb!c::valid-fun-use node
-                                (sb!c::specifier-type
+           (sb-c::valid-fun-use node
+                                (sb-c::specifier-type
                                  `(function ,args ,result)))))
-    (case (sb!c::combination-fun-source-name node)
+    (case (sb-c::combination-fun-source-name node)
       (logtest
        (if (or (valid-funtype '(signed-word signed-word) '*)
                (valid-funtype '(word word) '*))
@@ -324,8 +320,8 @@
            (values :default nil)))
       (logbitp
        (cond
-         ((or (valid-funtype '((constant-arg (mod #.n-word-bits)) signed-word) '*)
-              (valid-funtype '((constant-arg (mod #.n-word-bits)) word) '*))
+         ((or (valid-funtype `((constant-arg (mod ,n-word-bits)) signed-word) '*)
+              (valid-funtype `((constant-arg (mod ,n-word-bits)) word) '*))
           (values :transform '(lambda (index integer)
                                (%logbitp integer index))))
          (t (values :default nil))))
@@ -336,11 +332,11 @@
                                       ,type)
                                     'unsigned-byte)
                      (destructuring-bind (size posn integer)
-                         (sb!c::basic-combination-args node)
+                         (sb-c::basic-combination-args node)
                        (declare (ignore integer))
-                       (and (plusp (sb!c::lvar-value posn))
-                            (<= (+ (sb!c::lvar-value size)
-                                   (sb!c::lvar-value posn))
+                       (and (plusp (sb-c::lvar-value posn))
+                            (<= (+ (sb-c::lvar-value size)
+                                   (sb-c::lvar-value posn))
                                 width))))))
          (if (or (validp 'word (1- n-word-bits))
                  (validp 'signed-word (1- n-word-bits)))
@@ -371,9 +367,9 @@
                   :offset (tn-offset tn)))
 
 ;;; null-tn will be used for setting it, just check the lowtag
-#!+sb-thread
+#+sb-thread
 (defconstant pseudo-atomic-flag
-    (ash list-pointer-lowtag #!+little-endian 0 #!+big-endian 32))
-#!+sb-thread
+    (ash list-pointer-lowtag #+little-endian 0 #+big-endian 32))
+#+sb-thread
 (defconstant pseudo-atomic-interrupted-flag
-    (ash list-pointer-lowtag #!+little-endian 32 #!+big-endian 0))
+    (ash list-pointer-lowtag #+little-endian 32 #+big-endian 0))
