@@ -12,7 +12,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;;; utilities used during code generation
 
@@ -21,7 +21,7 @@
                                 (component *component-being-compiled*))
   (let* ((2comp (component-info component))
          (constants (ir2-component-constants 2comp)))
-    (ash (align-up (length constants) code-boxed-words-align) sb!vm:word-shift)))
+    (ash (align-up (length constants) code-boxed-words-align) sb-vm:word-shift)))
 
 ;;; the size of the NAME'd SB in the currently compiled component.
 ;;; This is useful mainly for finding the size for allocating stack
@@ -98,7 +98,7 @@
 ;;; * INLINE-CONSTANT-VALUE: given a canonical constant descriptor, computes
 ;;;    two values:
 ;;;     1. A label that will be used to emit the constant (usually a
-;;;         sb!assem:label)
+;;;         sb-assem:label)
 ;;;     2. A value that will be returned to code generators referring to
 ;;;         the constant (on x86oids, an EA object)
 ;;; * SORT-INLINE-CONSTANTS: Receives a vector of unique constants;
@@ -110,25 +110,25 @@
 ;;; * EMIT-INLINE-CONSTANT: receives a constant descriptor and its associated
 ;;;    label. Emits the constant.
 ;;;
-;;; Implementing this feature lets VOP generators use sb!c:register-inline-constant
-;;; to get handles (as returned by sb!vm:inline-constant-value) from constant
+;;; Implementing this feature lets VOP generators use sb-c:register-inline-constant
+;;; to get handles (as returned by sb-vm:inline-constant-value) from constant
 ;;; descriptors.
 ;;;
-#!+(or x86 x86-64 arm64)
+#+(or x86 x86-64 arm64)
 (defun register-inline-constant (&rest constant-descriptor)
   (declare (dynamic-extent constant-descriptor))
   (let ((asmstream *asmstream*)
-        (constant (sb!vm:canonicalize-inline-constant constant-descriptor)))
+        (constant (sb-vm:canonicalize-inline-constant constant-descriptor)))
     (ensure-gethash
      constant
      (asmstream-constant-table asmstream)
-     (multiple-value-bind (label value) (sb!vm:inline-constant-value constant)
+     (multiple-value-bind (label value) (sb-vm:inline-constant-value constant)
        (vector-push-extend (cons constant label)
                            (asmstream-constant-vector asmstream))
        value))))
-#!-(or x86 x86-64 arm64)
-(progn (defun sb!vm:sort-inline-constants (constants) constants)
-       (defun sb!vm:emit-inline-constant (&rest args)
+#-(or x86 x86-64 arm64)
+(progn (defun sb-vm:sort-inline-constants (constants) constants)
+       (defun sb-vm:emit-inline-constant (&rest args)
          (error "EMIT-INLINE-CONSTANT called with ~S" args)))
 ;;; Return T if and only if there were any constants emitted.
 (defun emit-inline-constants ()
@@ -136,12 +136,12 @@
          (constants (asmstream-constant-vector asmstream))
          (section (asmstream-data-section asmstream)))
     (when (plusp (length constants))
-      (dovector (constant (sb!vm:sort-inline-constants constants) t)
-        (sb!vm:emit-inline-constant section (car constant) (cdr constant))))))
+      (dovector (constant (sb-vm:sort-inline-constants constants) t)
+        (sb-vm:emit-inline-constant section (car constant) (cdr constant))))))
 
 ;;; If a constant is already loaded into a register use that register.
 (defun optimize-constant-loads (component)
-  (let* ((register-sb (sb-or-lose 'sb!vm::registers))
+  (let* ((register-sb (sb-or-lose 'sb-vm::registers))
          (loaded-constants
            (make-array (sb-size register-sb)
                        :initial-element nil)))
@@ -178,10 +178,10 @@
                             (remove-constant (tn-ref-load-tn ref))))))
                  (compatible-scs-p (a b)
                    (or (eql a b)
-                       (and (eq (sc-name a) 'sb!vm::control-stack)
-                            (eq (sc-name b) 'sb!vm::descriptor-reg))
-                       (and (eq (sc-name b) 'sb!vm::control-stack)
-                            (eq (sc-name a) 'sb!vm::descriptor-reg))))
+                       (and (eq (sc-name a) 'sb-vm::control-stack)
+                            (eq (sc-name b) 'sb-vm::descriptor-reg))
+                       (and (eq (sc-name b) 'sb-vm::control-stack)
+                            (eq (sc-name a) 'sb-vm::descriptor-reg))))
                  (find-constant-tn (constant sc)
                    (loop for (saved-constant . tn) across loaded-constants
                          when (and saved-constant
@@ -189,7 +189,7 @@
                                    (compatible-scs-p (tn-sc tn) sc))
                          return tn)))
           (case (vop-name vop)
-            ((move sb!vm::move-arg)
+            ((move sb-vm::move-arg)
              (let* ((args (vop-args vop))
                     (x (tn-ref-tn args))
                     (y (tn-ref-tn (vop-results vop)))
@@ -271,90 +271,39 @@
                    (format t
                            "missing generator for ~S~%"
                            (template-name (vop-info vop))))
-                  #!+arm64
+                  #+arm64
                   ((and (vop-next vop)
                         (eq (vop-name vop)
                             (vop-name (vop-next vop)))
-                        (memq (vop-name vop) '(move move-operand sb!vm::move-arg))
-                        (sb!vm::load-store-two-words vop (vop-next vop)))
+                        (memq (vop-name vop) '(move move-operand sb-vm::move-arg))
+                        (sb-vm::load-store-two-words vop (vop-next vop)))
                    (setf vop (vop-next vop)))
                   (t
                    (funcall gen vop)))))))
 
     ;; Truncate the final assembly code buffer to length
-    (sb!assem::truncate-section-to-length (asmstream-code-section asmstream))
+    (sb-assem::truncate-section-to-length (asmstream-code-section asmstream))
 
     (coverage-mark-lowering-pass component asmstream)
 
     (emit-inline-constants)
-    (let* ((fun-table)
-           (end-text (gen-label))
-           (start-fun-table (gen-label))
-           (n-entries (length (ir2-component-entries (component-info component))))
-           (fun-table-section
-            (let ((section (sb!assem::make-section)))
-              ;; Create space for the simple-fun offset table:
-              ;;  1 uint32 * N simple-funs
-              ;;  1 uint16 for the number of simple-funs
-              (emit section
-                    end-text `(.align 2) start-fun-table
-                    `(.skip ,(+ (* n-entries 4) 2)))
-              section))
-           (segment
-            (assemble-sections
-               (make-segment :header-skew
-                             (if (and (= code-boxed-words-align 1)
-                                      (oddp (length (ir2-component-constants
-                                                     (component-info component)))))
-                                 sb!vm:n-word-bytes
-                                 0)
-                             :run-scheduler (default-segment-run-scheduler)
-                             :inst-hook (default-segment-inst-hook))
-               (asmstream-data-section asmstream)
-               (asmstream-code-section asmstream)
-               (asmstream-elsewhere-section asmstream)
-               fun-table-section))
-           (octets
-            (segment-buffer segment))
-           (index
-            (label-position start-fun-table)))
-      (flet ((store-ub16 (index val)
-                 (multiple-value-bind (b0 b1)
-                     #!+little-endian
-                     (values (ldb (byte 8 0) val) (ldb (byte 8 8) val))
-                     #!+big-endian
-                     (values (ldb (byte 8 8) val) (ldb (byte 8 0) val))
-                   (setf (aref octets (+ index 0)) b0
-                         (aref octets (+ index 1)) b1)))
-             (store-ub32 (index val)
-                 (multiple-value-bind (b0 b1 b2 b3)
-                     #!+little-endian
-                     (values (ldb (byte 8  0) val) (ldb (byte 8  8) val)
-                             (ldb (byte 8 16) val) (ldb (byte 8 24) val))
-                     #!+big-endian
-                     (values (ldb (byte 8 24) val) (ldb (byte 8 16) val)
-                             (ldb (byte 8  8) val) (ldb (byte 8  0) val))
-                   (setf (aref octets (+ index 0)) b0
-                         (aref octets (+ index 1)) b1
-                         (aref octets (+ index 2)) b2
-                         (aref octets (+ index 3)) b3))))
-        (let ((padding (- index (label-position end-text))))
-          (unless (and (typep n-entries '(unsigned-byte 12))
-                       (typep padding '(unsigned-byte 4)))
-            (bug "Oversized code component?"))
-          ;; Assert that we are aligned for storing uint32_t
-          (aver (not (logtest index #b11)))
-          (dolist (entry (sb!c::ir2-component-entries
-                          (component-info component)))
-            (let ((val (label-position (entry-info-offset entry))))
-              (push val fun-table)
-              (store-ub32 index val)
-              (incf index 4)))
-          (store-ub16 index (logior (ash n-entries 4) padding)))
-        (aver (= (+ index 2) (length octets))))
-      (values segment (label-position end-text) fun-table
-              (asmstream-elsewhere-label asmstream)
-              (sb!assem::segment-fixup-notes segment)))))
+    (let* ((info (component-info component))
+           (simple-fun-labels
+            (mapcar #'entry-info-offset (ir2-component-entries info)))
+           (n-boxed (length (ir2-component-constants info)))
+           ;; Skew is either 0 or N-WORD-BYTES depending on whether the boxed
+           ;; header length is even or odd
+           (skew (if (and (= code-boxed-words-align 1) (oddp n-boxed))
+                     sb-vm:n-word-bytes
+                     0)))
+      (multiple-value-bind (segment text-length fixup-notes fun-table)
+          (assemble-sections asmstream
+                             simple-fun-labels
+                             (make-segment :header-skew skew
+                                           :run-scheduler (default-segment-run-scheduler)
+                                           :inst-hook (default-segment-inst-hook)))
+        (values segment text-length fun-table
+                (asmstream-elsewhere-label asmstream) fixup-notes)))))
 
 (defun label-elsewhere-p (label-or-posn kind)
   (let ((elsewhere (label-position *elsewhere-label*))
@@ -376,12 +325,12 @@
 ;;; FIXME: this pass runs even if no coverage instrumentation was generated.
 (defun coverage-mark-lowering-pass (component asmstream)
   (declare (ignorable component asmstream))
-  #!+(or x86-64 x86)
+  #+(or x86-64 x86)
   (let ((label (gen-label))
         ;; vector of lists of original source paths covered
         (src-paths (make-array 10 :fill-pointer 0))
         (previous-mark))
-    (dolist (buffer (reverse (sb!assem::section-buf-chain
+    (dolist (buffer (reverse (sb-assem::section-buf-chain
                               (asmstream-code-section asmstream))))
       (dotimes (i (length buffer))
         (let ((item (svref buffer i)))
