@@ -11,7 +11,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;; This phase runs before IR2 conversion, initializing each XEP's
 ;;; ENTRY-INFO structure. We call the VM-supplied
@@ -31,6 +31,26 @@
   (select-component-format component)
   (values))
 
+;;; An "effectively" null environment captures at most
+;;; a compilation policy, nothing more.
+;;; This is to make the 2nd value of FUNCTION-LAMBDA-EXPRESSION
+;;; accurate; but is it really important to do that? Nah.
+#+nil
+(defun effectively-null-lexenv-p (lexenv)
+  (or (null-lexenv-p lexenv)
+      (and (not (or (lexenv-funs lexenv)
+                    (lexenv-vars lexenv)
+                    (lexenv-blocks lexenv)
+                    (lexenv-tags lexenv)
+                    (lexenv-type-restrictions lexenv)
+                    (lexenv-lambda lexenv)
+                    (lexenv-cleanup lexenv)
+                    (lexenv-handled-conditions lexenv)
+                    (lexenv-disabled-package-locks lexenv)
+                    (lexenv-user-data lexenv)))
+           (or (not (lexenv-parent lexenv))
+               (null-lexenv-p (lexenv-parent lexenv))))))
+
 ;;; Initialize INFO structure to correspond to the XEP LAMBDA FUN.
 (defun compute-entry-info (fun info)
   (declare (type clambda fun) (type entry-info info))
@@ -43,11 +63,40 @@
     (setf (entry-info-offset info) (gen-label))
     (setf (entry-info-name info)
           (leaf-debug-name internal-fun))
-    (let ((doc (functional-documentation internal-fun))
+    (let ((form (functional-inline-expansion internal-fun))
+          (doc (functional-documentation internal-fun))
           (xrefs (pack-xref-data (functional-xref internal-fun))))
-      (setf (entry-info-info info) (if (and doc xrefs)
-                                       (cons doc xrefs)
-                                       (or doc xrefs))))
+      (setf (entry-info-form/doc/xrefs info)
+            (list (if (fasl-output-p *compile-object*)
+                      (and (policy bind (= store-source-form 3))
+                           ;; Downgrade the error to a warning if this was signaled
+                           ;; by SB-PCL::DONT-KNOW-HOW-TO-DUMP.
+                           ;; If not that, let the error propagate.
+                           (block nil
+                             (handler-bind
+                                 ((compiler-error
+                                   ;; Everything about error handling sucks so badly.
+                                   ;; Why turn a perfectly good error into a different
+                                   ;; and not necessarily better error?
+                                   (lambda (e &aux (c (encapsulated-condition e)))
+                                     ;; And why the heck isn't this just NO-APPLICABLE-METHOD
+                                     ;; on MAKE-LOAD-FORM?  Why did we choose to further obfuscate
+                                     ;; a condition that was reflectable and instead turn it
+                                     ;; into a dumb text string?
+                                     (when (and (typep c 'simple-error)
+                                                (search "know how to dump"
+                                                        (simple-condition-format-control c)))
+                                       ;; This might be worth a full warning. Dunno.
+                                       ;; After all, the user asked to do what can't be done.
+                                       (compiler-style-warn
+                                        "Can't preserve function source - ~
+missing MAKE-LOAD-FORM methods?")
+                                       (return nil)))))
+                               (constant-value (find-constant form)))))
+                      (and (policy bind (> store-source-form 0))
+                           form))
+                  doc
+                  xrefs)))
     (when (policy bind (>= debug 1))
       (let ((args (functional-arg-documentation internal-fun)))
         ;; When the component is dumped, the arglists of the entry
@@ -100,8 +149,12 @@
                         :%source-name (functional-%source-name ef)
                         :%debug-name (functional-%debug-name ef)
                         :lexenv (make-null-lexenv)))
-                  (closure (physenv-closure
-                            (lambda-physenv (main-entry ef)))))
+                  (main-entry (main-entry ef))
+                  (closure (and
+                            ;; It may have been deleted due to none of
+                            ;; the optional entries reaching it.
+                            (neq (functional-kind main-entry) :deleted)
+                            (physenv-closure (lambda-physenv main-entry)))))
              (dolist (ref (leaf-refs lambda))
                (let ((ref-component (node-component ref)))
                  (cond ((eq ref-component component))
@@ -112,7 +165,7 @@
                         (setf (ref-leaf ref) new)
                         (push ref (leaf-refs new))
                         (setf (leaf-refs lambda)
-                              (delq ref (leaf-refs lambda))))))))))
+                              (delq1 ref (leaf-refs lambda))))))))))
         (:toplevel
          (setq res t))))
     res))
