@@ -22,14 +22,14 @@
 
 (defconstant extended-symbol-size (1+ symbol-size))
 
-#!+sb-thread
+#+sb-thread
 (dolist (slot (primitive-object-slots
                (find 'thread *primitive-objects* :key #'primitive-object-name)))
   (when (slot-special slot)
     (setf (info :variable :wired-tls (slot-special slot))
           (ash (slot-offset slot) word-shift))))
 
-#!+gencgc
+#+gencgc
 (defconstant large-object-size
   (* 4 (max +backend-page-bytes+ gencgc-card-bytes
             gencgc-alloc-granularity)))
@@ -44,7 +44,7 @@
           (cdr entry)))
   `((bignum "unboxed" "bignum" "bignum")
     (ratio "boxed" "ratio_or_complex" "boxed")
-    (single-float ,(or #!+64-bit "immediate" "unboxed"))
+    (single-float ,(or #+64-bit "immediate" "unboxed"))
     (double-float "unboxed")
     (complex "boxed" "ratio_or_complex" "boxed")
     (complex-single-float "unboxed")
@@ -57,16 +57,16 @@
     ;; but it's only defined on non-x86 platforms for some reason.
     ;; The sizer is "lose" because it's an error if a function is encountered
     ;; in a heap scan.
-    (simple-fun ,(or #!+(or x86 x86-64) "lose" "fun_header") "lose" "lose")
+    (simple-fun ,(or #+(or x86 x86-64) "lose" "fun_header") "lose" "lose")
     ;; The closure scavenge function needs to know if the "self" slot
     ;; has pointer nature though it be fixnum tagged, as on x86.
     ;; The sizer is short_boxed.
-    (closure ,(or #!+(or x86 x86-64) "closure" "short_boxed") "lose" "short_boxed")
+    (closure ,(or #+(or x86 x86-64) "closure" "short_boxed") "lose" "short_boxed")
     ;; Like closure, but these can also have a layout pointer in the high header bytes.
-    (funcallable-instance ,(or #!+compact-instance-header "funinstance" "short_boxed")
+    (funcallable-instance ,(or #+compact-instance-header "funinstance" "short_boxed")
                           "lose" "short_boxed")
     ;; These have a scav and trans function, but no size function.
-    #!-(or x86 x86-64) (return-pc "return_pc_header" "return_pc_header" "lose")
+    #-(or x86 x86-64) (return-pc "return_pc_header" "return_pc_header" "lose")
 
     (value-cell "boxed")
     (symbol "tiny_boxed")
@@ -77,12 +77,12 @@
     (unbound-marker "immediate")
     (weak-pointer "lose" "weak_pointer" "boxed")
     (instance "instance" "lose" "short_boxed")
-    (fdefn ,(or #!+(or sparc arm) "boxed" "fdefn") "tiny_boxed")
+    (fdefn ,(or #+(or sparc arm riscv) "boxed" "fdefn") "tiny_boxed")
 
     (no-tls-value-marker "immediate")
 
-    #!+sb-simd-pack (simd-pack "unboxed")
-    #!+sb-simd-pack-256 (simd-pack-256 "unboxed")
+    #+sb-simd-pack (simd-pack "unboxed")
+    #+sb-simd-pack-256 (simd-pack-256 "unboxed")
     (filler "unboxed")
 
     (simple-array "boxed")
@@ -92,19 +92,19 @@
     (simple-array-unsigned-byte-8 "vector_unsigned_byte_8")
     (simple-array-unsigned-byte-15 "vector_unsigned_byte_16")
     (simple-array-unsigned-byte-16 "vector_unsigned_byte_16")
-    (simple-array-unsigned-fixnum #!-64-bit "vector_unsigned_byte_32"
-                                  #!+64-bit "vector_unsigned_byte_64")
+    (simple-array-unsigned-fixnum #-64-bit "vector_unsigned_byte_32"
+                                  #+64-bit "vector_unsigned_byte_64")
     (simple-array-unsigned-byte-31 "vector_unsigned_byte_32")
     (simple-array-unsigned-byte-32 "vector_unsigned_byte_32")
-    #!+64-bit (simple-array-unsigned-byte-63 "vector_unsigned_byte_64")
-    #!+64-bit (simple-array-unsigned-byte-64 "vector_unsigned_byte_64")
+    #+64-bit (simple-array-unsigned-byte-63 "vector_unsigned_byte_64")
+    #+64-bit (simple-array-unsigned-byte-64 "vector_unsigned_byte_64")
 
     (simple-array-signed-byte-8 "vector_unsigned_byte_8")
     (simple-array-signed-byte-16 "vector_unsigned_byte_16")
     (simple-array-signed-byte-32 "vector_unsigned_byte_32")
-    (simple-array-fixnum #!-64-bit "vector_unsigned_byte_32"
-                         #!+64-bit "vector_unsigned_byte_64")
-    #!+64-bit (simple-array-signed-byte-64 "vector_unsigned_byte_64")
+    (simple-array-fixnum #-64-bit "vector_unsigned_byte_32"
+                         #+64-bit "vector_unsigned_byte_64")
+    #+64-bit (simple-array-signed-byte-64 "vector_unsigned_byte_64")
 
     (simple-array-single-float "vector_unsigned_byte_32")
     (simple-array-double-float "vector_unsigned_byte_64")
@@ -116,8 +116,8 @@
 
     (simple-array-nil "vector_nil")
     (simple-base-string "base_string")
-    #!+sb-unicode (simple-character-string "character_string")
-    #!+sb-unicode (complex-character-string "boxed")
+    #+sb-unicode (simple-character-string "character_string")
+    #+sb-unicode (complex-character-string "boxed")
     (complex-base-string "boxed")
     (complex-vector-nil "boxed")
 
@@ -142,9 +142,9 @@
     (dolist (entry *scav/trans/size*)
       (when (string= (second entry) "unboxed")
         (setf bits (logior bits (ash 1 (ash (car entry) -2))))))
-    (format stream "static inline boolean unboxed_obj_widetag_p(unsigned char widetag) {~%")
-    #!+64-bit (format stream "  return (0x~XLU >> (widetag>>2)) & 1;" bits)
-    #!-64-bit (format stream "  int bit = widetag>>2;
+    (format stream "static inline boolean leaf_obj_widetag_p(unsigned char widetag) {~%")
+    #+64-bit (format stream "  return (0x~XLU >> (widetag>>2)) & 1;" bits)
+    #-64-bit (format stream "  int bit = widetag>>2;
   return (bit<32 ? 0x~XU >> bit : 0x~XU >> (bit-32)) & 1;"
                       (ldb (byte 32 0) bits) (ldb (byte 32 32) bits))
     (format stream "~%}~%"))
