@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; LIST and LIST*
 (define-vop (list-or-list*)
@@ -132,31 +132,6 @@
         (inst addu temp n-word-bytes))
       (align-csp temp))))
 
-(define-vop (allocate-code-object)
-  ;; BOXED is a count of words as a fixnum; it is therefore also a byte count
-  ;; as a raw value because n-fixnum-tag-bits = word-shift.
-  (:args (boxed :scs (any-reg))
-         (unboxed-arg :scs (any-reg)))
-  (:results (result :scs (descriptor-reg)))
-  (:temporary (:scs (non-descriptor-reg)) ndescr)
-  (:temporary (:scs (non-descriptor-reg)) unboxed)
-  (:temporary (:sc non-descriptor-reg :offset nl4-offset) pa-flag)
-  (:generator 100
-    (inst li ndescr (lognot lowtag-mask))
-    (inst srl unboxed unboxed-arg word-shift)
-    (inst addu unboxed unboxed lowtag-mask)
-    (inst and unboxed ndescr)
-    (inst sll ndescr boxed (- n-widetag-bits word-shift))
-    (inst or ndescr code-header-widetag)
-
-    (pseudo-atomic (pa-flag)
-      (inst or result alloc-tn other-pointer-lowtag)
-      (inst addu alloc-tn boxed)
-      (storew ndescr result 0 other-pointer-lowtag)
-      (storew unboxed-arg result code-code-size-slot other-pointer-lowtag)
-      (inst addu alloc-tn unboxed)
-      (storew null-tn result code-debug-info-slot other-pointer-lowtag))))
-
 (define-vop (make-fdefn)
   (:policy :fast-safe)
   (:translate make-fdefn)
@@ -259,9 +234,14 @@
   (:generator 6
     (inst addu bytes extra (* (1+ words) n-word-bytes))
     (inst sll header bytes (- n-widetag-bits n-fixnum-tag-bits))
-    (inst addu header header (+ (ash -2 n-widetag-bits) type))
-    (inst srl bytes bytes n-lowtag-bits)
-    (inst sll bytes bytes n-lowtag-bits)
+    ;; The specified EXTRA value is the exact value placed in the header
+    ;; as the word count when allocating code.
+    (cond ((= type code-header-widetag)
+           (inst addu header header type))
+          (t
+           (inst addu header header (+ (ash -2 n-widetag-bits) type))
+           (inst srl bytes bytes n-lowtag-bits)
+           (inst sll bytes bytes n-lowtag-bits)))
     (pseudo-atomic (pa-flag)
       (inst or result alloc-tn lowtag)
       (storew header result 0 lowtag)
