@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; Data object ref/set stuff.
 
@@ -31,7 +31,9 @@
   (:generator 1
     (storew value object offset lowtag)))
 
-(define-vop (init-slot set-slot))
+(define-vop (init-slot set-slot)
+  (:info name dx-p offset lowtag)
+  (:ignore name dx-p))
 
 ;;;; Symbol hacking VOPs:
 
@@ -241,11 +243,19 @@
   funcallable-instance-info-offset fun-pointer-lowtag
   (descriptor-reg any-reg) * %funcallable-instance-info)
 
-(define-vop (closure-ref slot-ref)
-  (:variant closure-info-offset fun-pointer-lowtag))
+(define-vop (closure-ref)
+  (:args (object :scs (descriptor-reg)))
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:info offset)
+  (:generator 4
+    (loadw value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
-(define-vop (closure-init slot-set)
-  (:variant closure-info-offset fun-pointer-lowtag))
+(define-vop (closure-init)
+  (:args (object :scs (descriptor-reg))
+         (value :scs (descriptor-reg any-reg)))
+  (:info offset)
+  (:generator 4
+    (storew value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
 (define-vop (closure-init-from-fp)
   (:args (object :scs (descriptor-reg)))
@@ -296,33 +306,37 @@
 
 ;;;; raw instance slot accessors
 
-(define-vop (raw-instance-ref/word)
-  (:translate %raw-instance-ref/word)
-  (:policy :fast-safe)
-  (:args (object :scs (descriptor-reg))
-         (index :scs (any-reg)))
-  (:arg-types * positive-fixnum)
-  (:results (value :scs (unsigned-reg)))
-  (:temporary (:scs (interior-reg)) lip)
-  (:result-types unsigned-num)
-  (:generator 5
-    (inst addu lip object index)
-    (loadw value lip instance-slots-offset instance-pointer-lowtag)))
+(macrolet ((def (suffix sc primtype)
+             `(progn
+                (define-vop (,(symbolicate "RAW-INSTANCE-REF/" suffix))
+                  (:translate ,(symbolicate "%RAW-INSTANCE-REF/" suffix))
+                  (:policy :fast-safe)
+                  (:args (object :scs (descriptor-reg))
+                         (index :scs (any-reg)))
+                  (:arg-types * positive-fixnum)
+                  (:results (value :scs (,sc)))
+                  (:temporary (:scs (interior-reg)) lip)
+                  (:result-types ,primtype)
+                  (:generator 5
+                    (inst addu lip object index)
+                    (loadw value lip instance-slots-offset instance-pointer-lowtag)))
 
-(define-vop (raw-instance-set/word)
-  (:translate %raw-instance-set/word)
-  (:policy :fast-safe)
-  (:args (object :scs (descriptor-reg))
-         (index :scs (any-reg))
-         (value :scs (unsigned-reg) :target result))
-  (:arg-types * positive-fixnum unsigned-num)
-  (:results (result :scs (unsigned-reg)))
-  (:temporary (:scs (interior-reg)) lip)
-  (:result-types unsigned-num)
-  (:generator 5
-    (inst addu lip object index)
-    (storew value lip instance-slots-offset instance-pointer-lowtag)
-    (move result value)))
+                (define-vop (,(symbolicate "RAW-INSTANCE-SET/" suffix))
+                  (:translate ,(symbolicate "%RAW-INSTANCE-SET/" suffix))
+                  (:policy :fast-safe)
+                  (:args (object :scs (descriptor-reg))
+                         (index :scs (any-reg))
+                         (value :scs (,sc) :target result))
+                  (:arg-types * positive-fixnum ,primtype)
+                  (:results (result :scs (,sc)))
+                  (:temporary (:scs (interior-reg)) lip)
+                  (:result-types ,primtype)
+                  (:generator 5
+                    (inst addu lip object index)
+                    (storew value lip instance-slots-offset instance-pointer-lowtag)
+                    (move result value))))))
+  (def word unsigned-reg unsigned-num)
+  (def signed-word signed-reg signed-num))
 
 (define-vop (raw-instance-ref/single)
   (:translate %raw-instance-ref/single)
