@@ -97,15 +97,17 @@
 
 ;;; SXHASH of FLOAT values is defined directly in terms of DEFTRANSFORM in
 ;;; order to avoid boxing.
-(deftransform sxhash ((x) (single-float))
+(defglobal +sxhash-single-float-expr+
   `(let ((bits (logand (single-float-bits x) ,(1- (ash 1 32)))))
      (logxor 66194023
              (sxhash (the fixnum
                           (logand sb-xc:most-positive-fixnum
                                   (logxor bits
                                           (ash bits -7))))))))
+(deftransform sxhash ((x) (single-float)) '#.+sxhash-single-float-expr+)
+
 #-64-bit
-(deftransform sxhash ((x) (double-float))
+(defglobal +sxhash-double-float-expr+
   `(let* ((hi (logand (double-float-high-bits x) ,(1- (ash 1 32))))
           (lo (double-float-low-bits x))
           (hilo (logxor hi lo)))
@@ -117,14 +119,15 @@
 
 ;;; SXHASH of FIXNUM values is defined as a DEFTRANSFORM because it's so
 ;;; simple.
-(deftransform sxhash ((x) (fixnum))
+(defglobal +sxhash-fixnum-expr+
   (let ((c (logand 1193941380939624010 sb-xc:most-positive-fixnum)))
     ;; shift by -1 to get sign bit into hash
     `(logand (logxor (ash x 4) (ash x -1) ,c) sb-xc:most-positive-fixnum)))
+(deftransform sxhash ((x) (fixnum)) '#.+sxhash-fixnum-expr+)
 
 ;;; Treat double-float essentially the same as a fixnum if words are 64 bits.
 #+64-bit
-(deftransform sxhash ((x) (double-float))
+(defglobal +sxhash-double-float-expr+
   ;; logical negation of magic constant ensures that 0.0d0 hashes to something
   ;; other than what the fixnum 0 hashes to (as tested in hash.impure.lisp)
   (let ((c (logandc1 1193941380939624010 sb-xc:most-positive-fixnum)))
@@ -135,47 +138,32 @@
        ;; but seriously, who changes that parameter?
        (logand (logxor (ash x 4) (ash x -2) ,c) sb-xc:most-positive-fixnum))))
 
+(deftransform sxhash ((x) (double-float)) '#.+sxhash-double-float-expr+)
+
 ;;; SXHASH of SIMPLE-BIT-VECTOR values is defined as a DEFTRANSFORM
 ;;; because it is endian-dependent.
+;;; ("because it is endian-dependent" is not a reason to define
+;;; a transform. This is probably better off as an asm routine)
 (deftransform sxhash ((x) (simple-bit-vector))
-  `(let ((result 410823708)
-         (length (length x)))
-    (declare (type fixnum result))
-    (cond
-      ((zerop length)
-       (mix result (sxhash 0)))
-      (t
-       (mixf result (sxhash length))
-       (multiple-value-bind (n-full-words n-bits-remaining)
-           (floor length sb-vm:n-word-bits)
-         (flet ((mix-into-result (num)
-                  (mixf result
-                        ,(ecase sb-c:*backend-byte-order*
-                                (:little-endian
-                                 '(logand num sb-xc:most-positive-fixnum))
-                                ;; FIXME: I'm not certain that
-                                ;; N-LOWTAG-BITS is the clearest way of
-                                ;; expressing this: it's essentially the
-                                ;; difference between `(UNSIGNED-BYTE
-                                ;; ,SB-VM:N-WORD-BITS) and (AND FIXNUM
-                                ;; UNSIGNED-BYTE).
-                                (:big-endian
-                                 '(ash num (- sb-vm:n-lowtag-bits)))))))
-           (declare (inline mix-into-result))
-           ;; FIXME: should we respect DEPTHOID?  SXHASH on strings
-           ;; doesn't seem to...
-           (dotimes (i n-full-words)
-             (mix-into-result (%vector-raw-bits x i)))
-           (if (zerop n-bits-remaining)
-               result
-               (mix-into-result
-                (logand (ash (1- (ash 1 n-bits-remaining))
-                             ,(ecase sb-c:*backend-byte-order*
-                                     (:little-endian 0)
-                                     (:big-endian
-                                      '(- sb-vm:n-word-bits
-                                        n-bits-remaining))))
-                        (%vector-raw-bits x n-full-words))))))))))
+  `(let* ((length (length x))
+          (result (word-mix 410823708 length)))
+    (declare (type unsigned-byte result))
+     (multiple-value-bind (n-full-words n-bits-remaining)
+         (floor length sb-vm:n-word-bits)
+       (dotimes (i n-full-words)
+         (setq result (word-mix (%vector-raw-bits x i) result)))
+       (when (plusp n-bits-remaining)
+         ;; FIXME: Do we really have to mask off bits of the final word?
+         ;; I don't think so, given that remaining bits are invariantly zero.
+         (setq result
+               (word-mix (logand (ash (1- (ash 1 n-bits-remaining))
+                                      ,(ecase sb-c:*backend-byte-order*
+                                         (:little-endian 0)
+                                         (:big-endian
+                                          '(- sb-vm:n-word-bits n-bits-remaining))))
+                                 (%vector-raw-bits x n-full-words))
+                         result)))
+       (logand result sb-xc:most-positive-fixnum))))
 
 ;;; Some other common SXHASH cases are defined as DEFTRANSFORMs in
 ;;; order to avoid having to do TYPECASE at runtime.
@@ -184,25 +172,15 @@
 ;;; strings, and of symbols whose names are known at compile time;
 ;;; except that since SXHASH on the cross-compilation host is not in
 ;;; general compatible with SXHASH on the target SBCL, we can't so
-;;; easily do this optimization in the cross-compiler.
-;;; To play it safe, we'll assert that SBCL itself would not benefit
-;;; from this optimization.
-(macrolet ((compiler-sxhash (x)
-             #+sb-xc-host `(error "Compiler wanted to eval (SXHASH '~S)" (lvar-value ,x))
-             #-sb-xc-host `(sxhash (lvar-value ,x))))
+;;; easily do this optimization in the cross-compiler - the SB-XC:SXHASH
+;;; operation will catch any attempt to call it on strings.
 (deftransform sxhash ((x) (string))
-  (cond ((constant-lvar-p x) (compiler-sxhash x))
-        ((csubtypep (lvar-type x) (specifier-type 'simple-string))
+  (cond ((csubtypep (lvar-type x) (specifier-type 'simple-string))
          '(%sxhash-simple-string x))
         (t
          '(%sxhash-string x))))
 (deftransform sxhash ((x) (symbol))
-  (cond ((csubtypep (lvar-type x) (specifier-type 'null))
-         ;; Test this before CONSTANT-LVAR-P because it does happen
-         ;; during cross-compilation, and we want to win, not lose.
-         (ash sb-vm::nil-value (- sb-vm:n-fixnum-tag-bits)))
-        ((constant-lvar-p x) (compiler-sxhash x))
-        ((csubtypep (lvar-type x) (specifier-type 'keyword))
+  (cond ((csubtypep (lvar-type x) (specifier-type 'keyword))
          ;; All interned symbols have a precomputed hash.
          ;; There's no way to ask the type system whether a symbol is known to
          ;; be interned, but we *can* test for the specific case of keywords.
@@ -220,10 +198,26 @@
             (if (= 0 result)
                 (ensure-symbol-hash x)
                 result)))))
-) ; end MACROLET
 
+;;; These transforms are somehow needed when compiling ARRAY-PSXHASH and
+;;; then never again.  Can't we define the guts of it to not need them?
 (deftransform psxhash ((x &optional depthoid) (character &optional t))
   `(char-code (char-upcase x)))
 
 (deftransform psxhash ((x &optional depthoid) (integer &optional t))
   `(sxhash x))
+
+;;; To define SB-XC:SXHASH compatibly without repeating the logic in the transforms
+;;; that define the numeric cases, we do some monkey business involving #. to paste
+;;; in the expressions that the transforms would return.
+#+sb-xc-host
+(progn
+  (defvar *sxhash-crosscheck* nil)
+  (defun sxhash (x)
+    (let ((answer (etypecase x ; croak on anything but these
+                    (null         (ash sb-vm::nil-value (- sb-vm:n-fixnum-tag-bits)))
+                    (fixnum       #.+sxhash-fixnum-expr+)
+                    (single-float #.+sxhash-single-float-expr+)
+                    (double-float #.+sxhash-double-float-expr+))))
+      (push (cons x answer) *sxhash-crosscheck*)
+      answer)))
