@@ -39,6 +39,7 @@
 #include "arch.h"
 #include "gc.h"
 #include "hopscotch.h"
+#include "code.h"
 #include "genesis/primitive-objects.h"
 #include "genesis/static-symbols.h"
 #include "genesis/layout.h"
@@ -59,6 +60,17 @@ os_vm_size_t dynamic_space_size = DEFAULT_DYNAMIC_SPACE_SIZE;
 os_vm_size_t thread_control_stack_size = DEFAULT_CONTROL_STACK_SIZE;
 
 sword_t (*scavtab[256])(lispobj *where, lispobj object);
+
+// "Transport" functions are responsible for deciding where to copy an object
+// and how many bytes to copy (usually the sizing function is inlined into the
+// trans function, so we don't call a sizetab[] entry), allocating memory,
+// and calling memcpy. These functions generally do not deposit forwarding
+// pointers into the original object - that would be up to the scav_x_pointer
+// function - however, there are some exceptions:
+//  - trans_list() is responsible for leaving FPs for all cons cells that
+//    it copied while laying out a chain of consecutive conses.
+//  - trans_code() is responsible for leaving FPs for both the code object
+//    AND all embedded functions.
 static lispobj (*transother[64])(lispobj object);
 sword_t (*sizetab[256])(lispobj *where);
 struct weak_pointer *weak_pointer_chain = WEAK_POINTER_CHAIN_END;
@@ -94,7 +106,8 @@ os_vm_size_t bytes_consed_between_gcs = 12*1024*1024;
 
 /* Medium-sized payload count is expressed in 15 bits. Objects in this category
  * may reside in immobile space: CLOSURE, INSTANCE, FUNCALLABLE-INSTANCE.
- * The single data bit is used as a closure's NAMED flag.
+ * The single data bit is used as a closure's NAMED flag,
+ * or an instance's "special GC strategy" flag.
  *
  * Header:  gen# |  data |     size |    tag
  *         -----   -----    -------   ------
@@ -325,10 +338,8 @@ trans_code(struct code *code)
 
     /* prepare to transport the code vector */
     lispobj l_code = (lispobj) LOW_WORD(code) | OTHER_POINTER_LOWTAG;
-    sword_t nheader_words = code_header_words(code->header);
-    sword_t ncode_words = code_unboxed_nwords(code->code_size);
     lispobj l_new_code = copy_large_object(l_code,
-                                           ALIGN_UP(nheader_words + ncode_words, 2),
+                                           code_total_nwords(code),
                                            CODE_PAGE_TYPE);
 
 #ifdef LISP_FEATURE_GENCGC
@@ -364,9 +375,7 @@ trans_code(struct code *code)
 #ifdef LISP_FEATURE_GENCGC
     /* Cheneygc doesn't need this os_flush_icache, it flushes the whole
        spaces once when all copying is done. */
-    os_flush_icache((os_vm_address_t) (((sword_t *)new_code) + nheader_words),
-                    ncode_words * sizeof(sword_t));
-
+    os_flush_icache(code_text_start(new_code), code_text_size(new_code));
 #endif
 
     gencgc_apply_code_fixups(code, new_code);
@@ -386,9 +395,7 @@ trans_code_header(lispobj object)
 static sword_t
 size_code_header(lispobj *where)
 {
-    struct code* c = (struct code*)where;
-    return ALIGN_UP(code_header_words(c->header) + code_unboxed_nwords(c->code_size),
-                    2);
+    return code_total_nwords((struct code*)where);
 }
 
 #ifdef RETURN_PC_WIDETAG
@@ -465,19 +472,36 @@ trans_fun_header(lispobj object)
  * instances
  */
 
+static inline lispobj copy_instance(lispobj object)
+{
+    // Object is an un-forwarded object in from_space
+    lispobj header = *(lispobj*)(object - INSTANCE_POINTER_LOWTAG);
+    lispobj copy = copy_object(object, 1 + (instance_length(header)|1));
+    set_forwarding_pointer(native_pointer(object), copy);
+    return copy;
+}
+
 static sword_t
 scav_instance_pointer(lispobj *where, lispobj object)
 {
     gc_dcheck(instancep(object));
-    lispobj header = *(lispobj*)(object - INSTANCE_POINTER_LOWTAG);
-    /* Object is a pointer into from space - not a FP. */
-    lispobj copy = copy_object(object, 1 + (instance_length(header)|1));
-
-    gc_dcheck(copy != object);
-
-    set_forwarding_pointer(native_pointer(object), copy);
+    lispobj copy = copy_instance(object);
     *where = copy;
 
+    struct instance* node = (struct instance*)(copy - INSTANCE_POINTER_LOWTAG);
+    // Copy chain of lockfree list nodes to consecutive memory addresses,
+    // just like trans_list does.  A logically deleted node will break the chain,
+    // as its 'next' will not satisfy instancep(), but that's ok.
+    if (node->header & CUSTOM_GC_SCAVENGE_FLAG) {
+        while (instancep(object = node->slots[INSTANCE_DATA_START]) // node.next
+               && from_space_p(object)
+               && !forwarding_pointer_p(native_pointer(object))) {
+            copy = copy_instance(object);
+            node->slots[INSTANCE_DATA_START] = copy;
+            node = (struct instance*)(copy - INSTANCE_POINTER_LOWTAG);
+        }
+    }
+
     return 1;
 }
 
@@ -680,9 +704,28 @@ scav_instance(lispobj *where, lispobj header)
         lbitmap = ((struct layout*)layout)->bitmap;
     }
     sword_t nslots = instance_length(header) | 1;
-    if (lbitmap == make_fixnum(-1))
+    if (lbitmap == make_fixnum(-1)) {
         scavenge(where+1, nslots);
-    else if (!fixnump(lbitmap)) {
+        return 1 + nslots;
+    }
+    // Specially scavenge the 'next' slot of a lockfree list node. If the node is
+    // pending deletion, 'next' will satisfy fixnump() but is in fact a pointer.
+    // GC doesn't care too much about the deletion algorithm, but does have to
+    // ensure liveness of the pointee, which may move unless pinned.
+    // One could imagine that the strategy is further determind by layout->_flags.
+    // A single bit suffices for now, but more generality is certainly possible.
+    if (header & CUSTOM_GC_SCAVENGE_FLAG) {
+        struct instance* node = (struct instance*)where;
+        lispobj next = node->slots[INSTANCE_DATA_START];
+        if (fixnump(next) && next) { // ignore initially 0 heap words
+            lispobj descriptor = next | INSTANCE_POINTER_LOWTAG;
+            scav1(&descriptor, descriptor);
+            // Fix the pointer but of course leave it in mid-deletion (untagged) state.
+            if (descriptor != (next | INSTANCE_POINTER_LOWTAG))
+                node->slots[INSTANCE_DATA_START] = descriptor & ~LOWTAG_MASK;
+        }
+    }
+    if (!fixnump(lbitmap)) {
         /* It is conceivable that 'lbitmap' points to from_space, AND that it
          * is stored in one of the slots of the instance about to be scanned.
          * If so, then forwarding it will deposit new bits into its first
@@ -750,7 +793,7 @@ static lispobj trans_bignum(lispobj object)
 
 /* Note: on the sparc we don't have to do anything special for fdefns, */
 /* 'cause the raw-addr has a function lowtag. */
-#if !defined(LISP_FEATURE_SPARC) && !defined(LISP_FEATURE_ARM)
+#if !defined(LISP_FEATURE_SPARC) && !defined(LISP_FEATURE_ARM) && !defined(LISP_FEATURE_RISCV)
 static sword_t
 scav_fdefn(lispobj *where, lispobj __attribute__((unused)) object)
 {
@@ -926,7 +969,7 @@ trans_weak_pointer(lispobj object)
         if (immobile_obj_gen_bits(native) == from_space) cell = broken; \
     }
 
-void scan_weak_pointers(void)
+void smash_weak_pointers(void)
 {
     struct weak_pointer *wp, *next_wp;
     for (wp = weak_pointer_chain; wp != WEAK_POINTER_CHAIN_END; wp = next_wp) {
@@ -1519,7 +1562,7 @@ lispobj *search_all_gc_spaces(void *pointer)
 /* Find the code object for the given pc, or return NULL on
    failure. */
 lispobj *
-component_ptr_from_pc(lispobj *pc)
+component_ptr_from_pc(char *pc)
 {
     lispobj *object = search_all_gc_spaces(pc);
 
@@ -1885,7 +1928,7 @@ scavenge_control_stack(struct thread *th)
                 gc_assert(n_words_scavenged == 1);
             }
         } else if (scavtab[header_widetag(object)] == scav_lose) {
-            lose("unboxed object in scavenge_control_stack: %p->%x, start=%p, end=%p",
+            lose("unboxed object in scavenge_control_stack: %p->%"OBJ_FMTX", start=%p, end=%p",
                  object_ptr, object, th->control_stack_start, access_control_stack_pointer(th));
         }
     }
