@@ -6,10 +6,7 @@ typedef struct ucontext os_context_t;
    wide-sigcontext for 32-bit kernels */
 typedef unsigned long os_context_register_t;
 
-static inline os_context_t *arch_os_get_context(void **void_context)
-{
-    return (os_context_t *) *void_context;
-}
+#include "arch-os-generic.inc"
 
 unsigned long os_context_fp_control(os_context_t *context);
 void os_restore_fp_control(os_context_t *context);
